;(function($){
    /**
     * 配置段管理
     * @param {String} [section]
     * @param {Object} [options]
     * @constructor
     */
    function Config(section, options) {
        options = options || {};
        $.extend(this._defaults, options.defaults);
        this.section(section, { config: options.config });
        return this;
    }
    function API(name) {
        //可使用$.mobile.path.parseUrl()进行替换$.url
        var regex = /^file:\/*/;
        var returnStr=this[name].search(regex) != -1 ? this[name].replace(regex, "") : this.baseURL+this[name]+"?";
        var token=sessionStorage.getItem("tk");
        if(token){
            token=JSON.parse(token);
            if(!!token.GIMAP_SID){
                returnStr += "GIMAP_SID="+token.GIMAP_SID+"&";
            }
            if(!!token.xykHyCurrentUserId){
                returnStr += "xykHyCurrentUserId="+token.xykHyCurrentUserId+"&";
            }
            if(!!token.currentJid){
                returnStr += "currentJid="+token.currentJid+"&";
            }
            if(!!token.openId){
                returnStr += "openId="+token.openId;
            }
        }
        return returnStr;
    }
    Config.prototype = {
        _defaults: (function(){
            var baseRoot = "paone-pir/xyk_hy/",
                buildURL = function(){
                    var args = [].slice.call(arguments);
                    return "http://" + args.join("/");
                };
            return {
                dev: {
                    debug: 1,

                    AjaxTimeout: 10000,
                    baseURL: '',
                    "oilSelf":"oilSelf.do", //自己吼油
                    "oilFriend":"oilFriend.do",//朋友帮吼
                    "getOilSelfHistoryList":"getOilSelfHistoryList.do",//获取吼油历史
                    "getbyUrl":"getbyUrl.do",//获取朋友帮吼URL
                    "checkMyOil":"checkMyOil.do",//查看油桶信息
                    "getLastOilResult":"getLastOilResult.do",//查询最后一次吼油结果
                    "checkMyOilDetail":"checkMyOilDetail.do",//查看油桶明细
                    "isTxtPaImType":"isTxtPaImType.do",  //渠道判断
                    "cardAddOil":"cardAddOil.do", //刮刮卡充值
                    "rechargeHistory":"queryCardOilList.do",//充值历史
                    API: API
                },
                stg: {
                    baseURL: buildURL("eim-talk-stg.dmzstg.pingan.com.cn", baseRoot)
                },
                prd: {
                    baseURL: buildURL("eim.pingan.com.cn", "rs/xyk_hy/")
                }
            };
        })(),
        /**
         * 切换配置段
         * @param {string} section
         * @param {object} [opts]
         * @returns {object} config
         * @example
         * 选择配置段
         * this.section('development');
         * 覆盖指定配置（通常用于程序运行时临时改变配置）
         * this.section('development',{config:{api:{login:'login.php'}}});
         */
        section: function (section, opts) {
            section = section || Object.keys(this._defaults)[0];
            opts = opts || {};

            //以继承方式合并到指定配置段
            var config = $.extend(true, {}, opts.config);
            for (var x in this._defaults) {
                config = $.extend(true, config, this._defaults[x]);
                if (x == section) break;
            }
            config.section = section;
            this._config = this._config || {};
            $.extend( this._config, config );
            return config;
        },
        /**
         *
         * @param [name]
         * @returns {*}
         */
        get: function (name) {
            return this._config[name] || this._config;
        },
        set: function (name, value) {
            this._defaults[this._config.section][name] = value;
            this._config[name] = value;
            return this;
        }
    };
    var config = window.config = new Config("stg"),
        cfg = config.get();

    //@todo 检查APP getconfig方法是否能正常调用
    /*App.getConfig(function (data) {//获取Native APP环境配置
        data = JSON.parse(data);
        var section = 'dev';
        switch (true) {
            case data["CONFIG_TAG"].indexOf("dev") > -1 :
                section = "dev";
                break;
            case data["CONFIG_TAG"].indexOf("stg") > -1 :
                section = "stg";
                break;
            case data["CONFIG_TAG"].indexOf("prd") > -1 :
                section = "prd";
                break;
        }
        config.section(section);
        window.DEBUG = cfg.debug && Boolean(cfg.section.indexOf('product') == -1);
        !DEBUG && console.warn('DEBUG is %s', DEBUG);
    });*/

    /**
     * 下面注册的ajax全局事件用来监视ajax行为
     */
    var _$doc = $(document);
    _$doc.ajaxSend(function(event, jqXHR, ajaxOptions){
        //allow abort for Ajax request during anytime
        $(this).one('abort.ajax', ajaxOptions, function(e){
            console.info('abort request of URL:'+e.data.url);
            //jqXHR.abort('The request has been aborted from URL:'+e.data.url);
            jqXHR.abort();
        });
    });
    _$doc.ajaxSuccess(function(event, XMLHttpRequest, ajaxOptions){
        console.info("status:"+ XMLHttpRequest.status +" statusText:"+ XMLHttpRequest.statusText +" responseText:"+ XMLHttpRequest.responseText + " url:"+ ajaxOptions.url);
    });
    _$doc.ajaxError(function(event, jqXHR, ajaxOptions, thrownError){
        console.error("status:"+ jqXHR.status +" statusText:"+ jqXHR.statusText +" thrown:"+ thrownError + " url:"+ ajaxOptions.url);
    });
    //所有ajax请求响应队列完成后触发
    _$doc.ajaxStop(function(event){
        $(this).off('abort.ajax');//abort.ajax事件的callbacks没有存在的意义。
    });
    $.ajaxSetup({ timeout:cfg.AjaxTimeout });

    /* mobile UI 配置 */
    //jqm配置
    $( document ).on( "mobileinit",  function () {
        console.log("configure jqm");
        //$.mobile.autoInitializePage = false;//禁止自动渲染页面组件
        //$.mobile.allowCrossDomainPages = true;//允许跨域页面载入当前页面dom中
        //$.mobile.ajaxEnabled = false;//禁用jqm的Ajax提交刷新行为
        //$.mobile.linkBindingEnabled = false;//使得jqm对元素自动绑定的click事件无效
        //$.mobile.hashListeningEnabled = false;//禁用jqm hash导航功能
        //$.mobile.pushStateEnabled = false;//禁用pushState功能，pushState是无刷新更改URL，属于HTML5 history对象的API

        // for loading widget
        $.mobile.loader.prototype.options.text = "请等待...";
        $.mobile.loader.prototype.options.textVisible = true;

        //$.mobile.defaultPageTransition = 'none';
        $.mobile.page.prototype.options.domCache = true;
    });

    seajs.config({
        base: "./js",
        // 别名配置
        alias: {
            'jquery': 'lib/jquery-2.1.0',
            "jqm":"lib/jquery.mobile.custom.min",
            "ui":"lib/custom.ui",
            "config": "config",
            "common": "common",
            "iScroll": "lib/iscroll-probe"
        },

        // 调试模式
        debug: true
    });
    /*window.checkVersion = function(){
        *//*App.call("isFans","10008@publicservice.pingan.com.cn",function(msg){
            sessionStorage.setItem("NATIVE_VERSION","true");
        });*//*
        var ua = navigator.userAgent.toLowerCase();
        var isPatxt = true;
        if(ua.indexOf("patxt") == -1){
            isPatxt = false;
        }
        if(isPatxt){
            if(App.IS_ANDROID && typeof window.android.isFans == "function"){
                sessionStorage.setItem("NATIVE_VERSION","true");
            }

            if(App.IS_IOS){
                var ua = navigator.userAgent.toUpperCase(),
                    reg = /APP\/\S+/g;

                if(!ua.match(reg)){
                    //如果ua里没有"APP"就return;
                    return;
                }
                var version  = ua.match(reg)[0].substring(4);
                if(version >= "1.1.9"){
                    //TODO:版本号到时需要改
                    sessionStorage.setItem("NATIVE_VERSION","true");
                }
            }
            var flag = App.IS_ANDROID && (!sessionStorage.getItem("NATIVE_VERSION"));
            sessionStorage.setItem("OLD_ANDROID",flag);
        }

    }*/
    /**
     * App.IS_TXT  是否为天下通
     * App.IS_IOS  是否为IOS终端
     * App.IS_ANDROID 是否为ANDROID终端
     * App.IS_NEW_VERSION 是否为天下通的新版本
     */
    window.checkVersion = function(){
        //判断是否为天下通
        if(App.IS_TXT){
            //根据isFans接口是否存在 判断是否为旧版本
            App.IS_NEW_VERSION = false
            if(App.IS_ANDROID && typeof window.android.isFans == "function"){
                App.IS_NEW_VERSION = true;
            }else if(App.IS_IOS){
                App.IS_NEW_VERSION = false
                App.call("isFans","10008@publicservice.pingan.com.cn",function(msg){
                    App.IS_NEW_VERSION = true;
                });
            }
        }
    }

    window.preLoadImg = function(){
        return;
    	//下载动画图片
    	var imgArray = ["images/roarOil/bg.png", "images/roarOil/fontIcon.png", "images/roarOil/icon.png", "images/roarOil/iconNum.png", "images/roarOil/newFontIcon.png", "images/dragon/dragon1.png", "images/dragon/dragon2.png", "images/dragon/dragon3.png", "images/dragon/fly.png"];
		//灰尘图片地址数组
		var dustImgArray = [];

		//灰尘对象数组
		var dustImgObjArray = []
		for( i = 1; i < 10; i++) {
			dustImgArray.push("images/dust/dust" + i + ".png");
		}
		
		//龙图片地址数组
		var dragonImgArray = [];
		
		//龙对象数组
		var dragonImgObjArray = []
		for( i = 1; i < 3; i++) {
			dragonImgArray.push("images/dragon/dragon" + i + ".png");
		}

		var img, imgIndex = 0, dustImgIndex = 0, dragonImgIndex = 0, imgLength = imgArray.length, dustImgLength = dustImgArray.length, dragonImgLength = dragonImgArray.length, allImgIndex = 0, allImgLength = imgLength + dustImgLength + dragonImgLength;

		//预加载所有图片
		function loadAllImg() {
			img = new Image();
			img.onload = function() {
				allImgIndex++;
			}
			img.src = imgArray[imgIndex]
			imgIndex++;
			if(imgIndex < imgLength) {
				loadAllImg();
			}
		}

		//预加载灰尘图片
		function loadDustImg() {
			img = new Image();
			img.onload = function() {
				allImgIndex++;
			}
			img.src = dustImgArray[dustImgIndex];
			dustImgObjArray[dustImgIndex] = img;
			if(dustImgIndex < dustImgLength - 1) {
				dustImgIndex++;
				loadDustImg();
			}
		}
		//预加载龙图片
		function loadDragonImg(){
			img = new Image();
			img.onload = function() {
				allImgIndex++;
			}
			img.src = dragonImgArray[dragonImgIndex];
			dragonImgObjArray[dragonImgIndex] = img;
			if(dragonImgIndex < dragonImgLength - 1) {
				dragonImgIndex++;
				loadDragonImg();
			}
		}
		
		loadAllImg();
		loadDustImg();
		loadDragonImg();
    }

    //微信分享的设置
    var onBridgeReady = function(){
        WeixinJSBridge.on('menu:share:timeline',function(args){
            WeixinJSBridge.invoke('shareTimeline',{
                "appid":"",
                "img_url":"http://pinganpub.u.qiniudn.com/100060140057586094628661.png",
                "img_width":"120",
                "img_height":"120",
                "link":"http://eim-talk-stg.dmzstg.pingan.com.cn/paone-pir/xyk_hy/txtPrivilege.html",
                "title":"平安信用卡,吼出免费油",
                "desc":"平安信用卡,吼出免费油"
            });
        });
        WeixinJSBridge.on("menu:share:appmessage",function(args){
            WeixinJSBridge.invoke("sendAppMessage",{
                "appid":"",
                "img_url":"http://pinganpub.u.qiniudn.com/100060140057586094628661.png",
                "img_width":"120",
                "img_height":"120",
                "link":"http://eim-talk-stg.dmzstg.pingan.com.cn/paone-pir/xyk_hy/txtPrivilege.html",
                "title":"平安信用卡,吼出免费油",
                "desc":"平安信用卡,吼出免费油"
            })
        });
        WeixinJSBridge.on("menu:share:weibo",function(args){
            WeixinJSBridge.invoke("shareWeibo",{
                "url":"http://eim-talk-stg.dmzstg.pingan.com.cn/paone-pir/xyk_hy/txtPrivilege.html",
                "content":"平安信用卡,吼出免费油"
            })
        });
    }
    if(document.addEventListener){
        document.addEventListener("WeixinJSBridgeReady",onBridgeReady,false);
    }

    if(!localStorage.randomIndex){
        localStorage.randomIndex = 0;
    }

    window.onunload = function(){
        var loadArr = sessionStorage.getItem("URL_ARRAY");
        if(loadArr && typeof loadArr === 'string'){
            loadArr = JSON.parse(loadArr);
            loadArr.push(location.href);
        }else{
            loadArr = [];
        }
    }

    window.storeURL = function (changeURL){
        var urlArr = sessionStorage.getItem("URL_HISTORY");
        if( urlArr && typeof urlArr === "string"){
            try{
                urlArr = JSON.parse(urlArr);
            }catch(e){
                urlArr = [urlArr]
            }
            urlArr.push(changeURL||location.href);
            sessionStorage.setItem("URL_HISTORY",JSON.stringify(urlArr));
        }else{
            urlArr = [];
            urlArr.push(location.href);
            sessionStorage.setItem("URL_HISTORY",JSON.stringify(urlArr));
        }
    }
    storeURL();
    window.nativeCallBack = function(num){
        if(num == '1'){
            var arr = sessionStorage.getItem("URL_HISTORY");
            if(arr && typeof arr === 'string'){
                arr = JSON.parse(arr)
                if(arr instanceof Array && arr.length>=2){
                    arr.pop();
                    arr.pop();
                    sessionStorage.setItem("URL_HISTORY",JSON.stringify(arr));
                }else{
                      arr = [];
                    sessionStorage.setItem("URL_HISTORY",JSON.stringify(arr));
                }
            }
        }else{
            var arr = sessionStorage.getItem("URL_HISTORY");
            if(arr && typeof arr === 'string'){
                try{
                    arr = JSON.parse(arr);
                    if(arr instanceof Array && arr.length>=2){
                        arr.pop();
                        var backUrl = arr.pop();
                        sessionStorage.setItem("URL_HISTORY",JSON.stringify(arr));
                        location.href = backUrl;
                    }else{
                        App.call("backnative");
                    }
                }catch(e){
                    App.call("backnative");
                }
            }
        }
    }

    window.localStorage.storageVersion = "003";
    var cacheList = [
        "../addOilByCard.html",
        "../css/common.css",
        "../css/index.css",
        "../css/jqm.popup.css",
        "../css/jquery.mobile.css",
        "../css/public.css",
        "../css/roarOil.css",
        "../css/yellBySelf.css",
        "../images/ajax-loader.gif",
        "../images/dragon/dragon1.png",
        "../images/dragon/dragon2.png",
        "../images/dragon/dragon3.png",
        "../images/dragon/fly.png",
        "../images/dust/dust.png",
        "../images/dust/dust1.png",
        "../images/dust/dust2.png",
        "../images/dust/dust3.png",
        "../images/dust/dust4.png",
        "../images/dust/dust5.png",
        "../images/dust/dust6.png",
        "../images/dust/dust7.png",
        "../images/dust/dust8.png",
        "../images/dust/dust9.png",
        "../images/gasTips/ad.jpg",
        "../images/gasTips/credit.jpg",
        "../images/gasTips/list_tips.png",
        "../images/roarOil/bg.png",
        "../images/roarOil/bg1.png",
        "../images/roarOil/bg2.png",
        "../images/roarOil/btn.png",
        "../images/roarOil/cr.png",
        "../images/roarOil/fontIcon.png",
        "../images/roarOil/hero.png",
        "../images/roarOil/icon.png",
        "../images/roarOil/iconNum.png",
        "../images/roarOil/indexIcon.png",
        "../images/roarOil/lt.png",
        "../images/roarOil/newFontIcon.png",
        "../images/roarOil/pop1Bg.png",
        "../images/roarOil/resultBg.png",
        "../images/roarOil/view1Bg1.png",
        "../images/roarOil/view1Bg2.png",
        "../images/roarOil/xz.png",
        "../index.html",
        "../inviteToYell.html",
        "../js/addOilByCard.js",
        "../indexModule/js/common.js",
        "../indexModule/js/config.js",
        "../indexModule/js/index.js",
        "../js/inviteToYell.js",
        "../js/lib/custom.ui.js",
        "../js/lib/iscroll-probe.js",
        "../js/lib/jqm.popup.js",
        "../js/lib/jqm.tab.js",
        "../js/lib/jquery-2.1.0.min.js",
        "../js/lib/jquery-2.1.0.min.map",
        "../js/lib/jquery.mobile.custom.min.js",
        "../js/lib/sea.min.js",
        "../js/lib/seajs-style-debug.js",
        "../js/lib/seajs-style.js",
        "../indexModule/js/native.js",
        "../js/roarAnimate.js",
        "../js/roarOil.js",
        "../js/yellByFriend.js",
        "../js/yellBySelf.js",
        "../js/yellHistory.js",
        "../js/yellingRecord.js",
        "../js/YellOutModel.js",
        "../link.html",
        "../oilDiscount.html",
        "../txtPrivilege.html",
        "../yellBible.html",
        "../yellByFriend.html",
        "../yellBySelf.html",
        "../yellHistory.html",
        "../yellingRecord.html"
    ];
    window.localStorage.cacheList = JSON.stringify(cacheList);
})(jQuery);;