//为ajax增加请求重试功能
! function($) {
    var originalAjax = $.ajax,
        setReplay = function(XHR, ctx){
            XHR.replay = function() {
                return $.ajax(ctx);
            };
        };

    $.ajax = function(ctx) {
        return originalAjax.apply(this, arguments).done(function(data, textStatus, XHR) {
            setReplay(XHR, ctx);
        }).fail(function(XHR){
            setReplay(XHR, ctx);
        });
    };
}($);
/**
 * 序列化表单
 * @returns {JSON}
 */
$.fn.serializeJSON = function(){
    var data = {};
    $.each($(this).serializeArray(), function(idx, obj){
        data[ obj.name ] = obj.value;
    });
    return data;
}
/**
 * load script file 加载js脚本，如果不需要requireJS的话
 * @param url
 * @param options
 * @returns {*}
 */
$.cachedScript = function( url, options ) {
    // Allow user to set any option except for dataType, cache, and url
    options = $.extend( options || {}, {
        dataType: "script",
        cache: true,
        url: url
    });

    // Use $.ajax() since it is more flexible than $.getScript
    // Return the jqXHR object so we can chain callbacks
    return jQuery.ajax( options );
};
/**
 * 单纯的占位符替换 数据{name: "value"} 模版{{name}} 输出value
 * @param {String} tpl
 * @param {JSON} data
 * @returns {String}
 */
function template(tpl, data){
    $.each(data, function(i, e){
        tpl = tpl.replace(RegExp("{{"+ i +"}}","g"), e);
    })
    return tpl;
}
/**
 * http错误处理，返回实例化后的Error对象
 * @param {jqXHR|XMLHttpRequest} jqXHR
 * @returns {Error}
 */
function httpErrorHandle(jqXHR){
    var status = jqXHR.status,
        statusText = jqXHR.statusText,
        msg;

    switch ( true ) {
        case status == 408 || statusText == "timeout":
            msg = "网络超时";
            break
        case 500 < status && status < 505:
            msg = "服务器错误";
            break;
        case statusText == "error":
            msg = "无网络";
            break
        default:
            msg = "HTTP请求："+jqXHR.statusText;
            break;
    }
    return new Error(msg);
}

/**
 * 获取URL参数
 * @return {String} name
 */
function getParams(name) {
    var reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i');
    var r = window.location.search.substr(1).match(reg);
    if(r) {
        return decodeURI(r[2]);
    }
    return null;
}

//事件模型
var Events = {
    _events : null,
    _eveSplit : /\s+/,
    _nsSplit : /[.:]/,
    /**
     * 事件匹配
     * @param eve 准备触发的事件
     * @param match 已绑定的事件
     * @returns {boolean}
     * @private
     */
    _isMatchEve : function(strEve, strMatch) {
        var arrEve = this._split(strEve, this._nsSplit);
        var arrMatch = this._split(strMatch, this._nsSplit);

        /*
         事件判断逻辑
         event    == event
         event    == event.ns
         event.ns == event.ns
         .ns      == event.ns
         menu:share:friend == menu:share:friend
         */
        if ( arrEve.length == 1 ) {
            return arrEve[0] == arrMatch[0];
        } else if ( arrEve.length == 2 ) {
            return arrEve[0] ? strEve == strMatch : arrEve[1] == arrMatch[1];
        } else if ( arrEve.length == arrMatch.length ) {
            return strEve == strMatch;
        }

        return false;
    },
    _split : function(name, regex) {
        return name.trim().split(regex)
    },
    on : function(name, callback, context) {
        this._events || (this._events = {});

        var arrName = this._split(name, this._eveSplit);
        arrName.forEach(function(eve) {
            var events = this._events[eve] || (this._events[eve] = []);
            events.push({
                callback : callback,
                ctx : context || this
            });
        }, this);
        return this;
    },
    trigger : function(name) {
        if(!this._events)
            return this;
        var events = [], args = [].slice.call(arguments, 1);

        this._split(name, this._eveSplit).forEach(function(eve) {
            for(var key in this._events) {
                if ( !this._events.hasOwnProperty(key) ) continue;
                this._isMatchEve(eve, key) && ( events = events.concat(this._events[key]));
            }
        }, this);

        events.forEach(function(e) {
            e.callback.apply(e.ctx, args)
        });
        return this;
    },
    once : function(name, callback, context) {
        var self = this, once = function() {
            self.off(name, once);
            callback.apply(this, arguments);
        };
        this._split(name, this._eveSplit).forEach(function(eve) {
            this.on(eve, once, context);
        }, this);
        return this;
    },
    off : function(name, callback, context) {
        if(!this._events)
            return this;

        if(!name && !callback && !context) {
            this._events = {};
            return this;
        } typeof callback == "object" && ( context = callback);

        var self = this, args = arguments, key, remove = function(e, k, events) {
            var or = args.length == 2 && (e.callback === callback || e.ctx === context), and = args.length == 3 && (e.callback === callback && e.ctx === context); (or || and ) && events.splice(k, 1);
            return events.length == 0;
        };

        this._split(name, this._eveSplit).forEach(function(eve) {
            for(key in this._events) {
                if(!this._events.hasOwnProperty(key) || !this._isMatchEve(eve, key))
                    continue;
                args.length == 1 ?
                    delete self._events[key] : self._events[key].some(remove) &&
                delete this._events[key];
            }
        }, this);
        return this;
    }
};

/**
 * 百度经纬度转换成国测局经纬度
 * @param lng 百度经度
 * @param lat 百度纬度
 * @returns {{lat: number, lng: number}}
 */

function bdGPS2gcjGPS(lng, lat){
    var x_pi = 3.14159265358979324 * 3000.0 / 180.0;
    var x = lng - 0.0065, y = lat - 0.006,
        z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * x_pi);
    var theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * x_pi);

    lng = z * Math.cos(theta);
    lat = z * Math.sin(theta);

    return {lat: lat, lng: lng};
}

function gcjGPS2bdGPS(lng, lat){
    var x_pi = 3.14159265358979324 * 3000.0 / 180.0;
    var x = lng, y = lat,
        z = Math.sqrt(x * x + y * y) + 0.00002 * Math.sin(y * x_pi);
    var theta = Math.atan2(y, x) + 0.000003 * Math.cos(x * x_pi);

    lng = z * Math.cos(theta) + 0.0065;
    lat = z * Math.sin(theta) + 0.006;

    return {lat: lat, lng: lng};
}

Array.prototype.shuffle = function shuffle(o){
    o = o || this;
    for(var j, x, i = o.length; i; j = Math.floor(Math.random() * i), x = o[--i], o[i] = o[j], o[j] = x);
    return o;
}

function isEmpty(value) {
    switch (true) {
        case typeof value == "number":
            return !parseFloat(value);
            break;
        case (Array.isArray(value) || typeof value == "string"):
            return value.length == 0;
            break;
        case value instanceof Object:
            return JSON.stringify(value) == "{}";
            break;
        default:
            return !value;
            break;
    }
}

/**
 * 获取URL参数
 * @return {String} name
 */
function getParams (name, href) {
    var reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i');
    var search = href ? "?" + href.split("?")[1] : window.location.search;
    var r = search.substr(1).match(reg);
    if(r) {
        return decodeURI(r[2]);
    }
    return null;
}

/**
 * Creates a function that will delay the execution of `func` until after
 * `wait` milliseconds have elapsed since the last time it was invoked. Pass
 * an `options` object to indicate that `func` should be invoked on the leading
 * and/or trailing edge of the `wait` timeout. Subsequent calls to the debounced
 * function will return the result of the last `func` call.
 *
 * Note: If `leading` and `trailing` options are `true`, `func` will be called
 * on the trailing edge of the timeout only if the the debounced function is
 * invoked more than once during the `wait` timeout.
 *
 * @static
 * @memberOf _
 * @category Functions
 * @param {Function} func The function to debounce.
 * @param {Number} wait The number of milliseconds to delay.
 * @param {Object} [options] The options object.
 *  [leading=false] A boolean to specify execution on the leading edge of the timeout.
 *  [maxWait] The maximum time `func` is allowed to be delayed before it's called.
 *  [trailing=true] A boolean to specify execution on the trailing edge of the timeout.
 * @returns {Function} Returns the new debounced function.
 * @example
 *
 * var lazyLayout = _.debounce(calculateLayout, 300);
 * jQuery(window).on('resize', lazyLayout);
 *
 * jQuery('#postbox').on('click', _.debounce(sendMail, 200, {
     *   'leading': true,
     *   'trailing': false
     * });
 */
function debounce(func, wait, options) {
    var args,
        result,
        thisArg,
        callCount = 0,
        lastCalled = 0,
        maxWait = false,
        maxTimeoutId = null,
        timeoutId = null,
        trailing = true,
        nativeMax = Math.max;

    function clear() {
        clearTimeout(maxTimeoutId);
        clearTimeout(timeoutId);
        callCount = 0;
        maxTimeoutId = timeoutId = null;
    }

    function delayed() {
        var isCalled = trailing && (!leading || callCount > 1);
        clear();
        if (isCalled) {
            if (maxWait !== false) {
                lastCalled = new Date;
            }
            result = func.apply(thisArg, args);
        }
    }

    function maxDelayed() {
        clear();
        if (trailing || (maxWait !== wait)) {
            lastCalled = new Date;
            result = func.apply(thisArg, args);
        }
    }

    wait = nativeMax(0, wait || 0);
    if (options === true) {
        var leading = true;
        trailing = false;
    } else if (typeof (options) == "object") {
        leading = options.leading;
        maxWait = 'maxWait' in options && nativeMax(wait, options.maxWait || 0);
        trailing = 'trailing' in options ? options.trailing : trailing;
    }
    return function() {
        args = arguments;
        thisArg = this;
        callCount++;

        // avoid issues with Titanium and `undefined` timeout ids
        // https://github.com/appcelerator/titanium_mobile/blob/3_1_0_GA/android/titanium/src/java/ti/modules/titanium/TitaniumModule.java#L185-L192
        clearTimeout(timeoutId);

        if (maxWait === false) {
            if (leading && callCount < 2) {
                result = func.apply(thisArg, args);
            }
        } else {
            var now = new Date;
            if (!maxTimeoutId && !leading) {
                lastCalled = now;
            }
            var remaining = maxWait - (now - lastCalled);
            if (remaining <= 0) {
                clearTimeout(maxTimeoutId);
                maxTimeoutId = null;
                lastCalled = now;
                result = func.apply(thisArg, args);
            }
            else if (!maxTimeoutId) {
                maxTimeoutId = setTimeout(maxDelayed, remaining);
            }
        }
        if (wait !== maxWait) {
            timeoutId = setTimeout(delayed, wait);
        }
        return result;
    };
}
/**** Date ****/
/*
 函数：格式化日期
 参数：formatStr-格式化字符串
 d：将日显示为不带前导零的数字，如1
 dd：将日显示为带前导零的数字，如01
 ddd：将日显示为缩写形式，如Sun
 dddd：将日显示为全名，如Sunday
 M：将月份显示为不带前导零的数字，如一月显示为1
 MM：将月份显示为带前导零的数字，如01
 MMM：将月份显示为缩写形式，如Jan
 MMMM：将月份显示为完整月份名，如January
 yy：以两位数字格式显示年份
 yyyy：以四位数字格式显示年份
 h：使用12小时制将小时显示为不带前导零的数字，注意||的用法
 hh：使用12小时制将小时显示为带前导零的数字
 H：使用24小时制将小时显示为不带前导零的数字
 HH：使用24小时制将小时显示为带前导零的数字
 m：将分钟显示为不带前导零的数字
 mm：将分钟显示为带前导零的数字
 s：将秒显示为不带前导零的数字
 ss：将秒显示为带前导零的数字
 l：将毫秒显示为不带前导零的数字
 ll：将毫秒显示为带前导零的数字
 tt：显示am/pm
 TT：显示AM/PM
 返回：格式化后的日期
 */
Date.prototype.format = function (formatStr) {
    var date = this;
    /*
     函数：填充0字符
     参数：value-需要填充的字符串, length-总长度
     返回：填充后的字符串
     */
    var zeroize = function (value, length) {
        if (!length) {
            length = 2;
        }
        value = String(value);
        for (var i = 0, zeros = ''; i < (length - value.length); i++) {
            zeros += '0';
        }
        return zeros + value;
    };
    return (formatStr||"").replace(/"[^"]*"|'[^']*'|\b(?:d{1,4}|M{1,4}|yy(?:yy)?|([hHmstT])\1?|[lLZ])\b/g, function($0) {
        switch ($0) {
            case 'd': return date.getDate();
            case 'dd': return zeroize(date.getDate());
            case 'ddd': return ['Sun', 'Mon', 'Tue', 'Wed', 'Thr', 'Fri', 'Sat'][date.getDay()];
            case 'dddd': return ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'][date.getDay()];
            case 'M': return date.getMonth() + 1;
            case 'MM': return zeroize(date.getMonth() + 1);
            case 'MMM': return ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][date.getMonth()];
            case 'MMMM': return ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'][date.getMonth()];
            case 'yy': return String(date.getFullYear()).substr(2);
            case 'yyyy': return date.getFullYear();
            case 'h': return date.getHours() % 12 || 12;
            case 'hh': return zeroize(date.getHours() % 12 || 12);
            case 'H': return date.getHours();
            case 'HH': return zeroize(date.getHours());
            case 'm': return date.getMinutes();
            case 'mm': return zeroize(date.getMinutes());
            case 's': return date.getSeconds();
            case 'ss': return zeroize(date.getSeconds());
            case 'l': return date.getMilliseconds();
            case 'll': return zeroize(date.getMilliseconds());
            case 'tt': return date.getHours() < 12 ? 'am' : 'pm';
            case 'TT': return date.getHours() < 12 ? 'AM' : 'PM';
        }
    });
};
