;(function($){
    /**
     * 配置段管理
     * @param {String} [section]
     * @param {Object} [options]
     * @constructor
     */
    function Config(options) {
        var section, lastSec;
        options = options || {};
        section = options.section; $.isPlainObject(options) && $.extend(this._defaults, options.defaults);
        lastSec = Object.keys(this._defaults).pop();

        //通过当前加载域名判断配置段
        $.each(this._defaults, function(sec, opts) {
            if(opts.baseURL.match(new RegExp("^https?://" + location.hostname)) != null) {
                section = section || sec;
                //优先使用定义的配置段
                return false;
            }
        });
        this.section(section || lastSec, {
            config : options.config
        });
        window.DEBUG = Boolean(this._config.debug && Boolean(this._config.section.indexOf(lastSec) == -1));
        return this;
    }

    function API(name) {
        //可使用$.mobile.path.parseUrl()进行替换$.url
        var regex = /^abs:\/*/, val = this[name] || "";
        return val.search(regex) != -1 ? val.replace(regex, "") : this.baseURL + val;
    }


    Config.prototype = {
        _defaults : {
            dev : {},
            stg : {},
            prd : {}
        },
        /**
         * 切换配置段
         * @param {string} section
         * @param {object} [opts]
         * @returns {object} config
         * @example
         * 选择配置段
         * this.section('development');
         * 覆盖指定配置（通常用于程序运行时临时改变配置）
         * this.section('development',{config:{api:{login:'login.php'}}});
         */
        section : function(section, opts) {
            //todo 自动追加API方法
            section = section || Object.keys(this._defaults)[0];
            opts = opts || {};

            //以继承方式合并到指定配置段
            var config = $.extend(true, {}, opts.config);
            for(var x in this._defaults) {
                config = $.extend(true, config, this._defaults[x]);
                if(x == section)
                    break;
            }
            config.section = section;
            this._config = this._config || {};
            $.extend(this._config, config);
            return config;
        },
        /**
         *
         * @param [name]
         * @returns {*}
         */
        get : function(name) {
            return this._config[name] || this._config;
        },
        set : function(name, value) {
            this._defaults[this._config.section][name] = value;
            this._config[name] = value;
            return this;
        }
    };

    var config = {
        //section: "stg",
        defaults: (function(){
            var baseRoot = "cms-tmplt/pinganlife/",
                buildURL = function(){
                    var args = [].slice.call(arguments);
                    return "http://" + args.join("/");
                };
            return {
                dev: {
                    debug: 1,

                    BMapAK: "rKRkWTB2lPEUCIQrFTMeeuLd",
                    geocoder: "http://api.map.baidu.com/geocoder/v2/",

                    AjaxTimeout: 5000,
                    filter: ["平安生活"],//用来过滤数据的关键词数组

                    queryShopDetail: "queryShopDetail.do",
                    queryHuodongDetail : "queryHuodongDetail.do",
                    queryHuodongList: "queryHuodongList.do",
                    queryOtherShopList: "queryOtherShopList.do",

                    creditCardBind: "http://www.baidu.com",
                    creditCardApply: "abs:https://wap-ebank.pingan.com/weixin/modules/online_apply_card/index.html?cid=txt#/showCards",

                    //获取客户车主类信用卡在“加油88折”返现记录信息接口
                    creditCardService: {
                        getRefuelDiscountInfo: "abs:data/getRefuelDiscountInfo.json",
                        checkUser: "abs:data/checkUserType.json",
                        API: API
                    },

                    queryShopList: "abs:data/index.json",
                    //creditCardBaseInfo: "data/creditBaseInfo.json",
                    queryBanner: "abs:data/queryBanner.json",

                    baseURL: 'http://localhost',
                    API: API
                },
                stg: {
                    queryShopList: "queryShopList.do",
                    queryShopDetail: "queryShopDetail.do",
                    queryHuodongDetail : "queryHuodongDetail.do",
                    queryHuodongList: "queryHuodongList.do",
                    queryOtherShopList: "queryOtherShopList.do",
                    //baseURL: buildURL("stg.pingan.com.cn", baseRoot)
                    baseURL: buildURL("www.pingan.com", baseRoot),//query*系列接口取用银行那边的生产数据测试，下面测试环境接口不变

                    creditCardBaseInfo: "http://test-public-txt.pingan.com.cn:10080/HMP/rest/getExemptToken?appid=10008&url=http%3A%2F%2Feim-talk-stg.dmzstg.pingan.com.cn%2Fzpa-pir%2FxykQueryBusiness%3FweappNo%3DPAXYK95511_06",
                    creditCardBind:"http://test-public-txt.pingan.com.cn:10080/HMP/rest/getExemptToken?appid=10008&url=http%3A%2F%2Feim-talk-stg.dmzstg.pingan.com.cn%2Fzpa-pir%2FxykFreeLogin%3FweappNo%3DPAXYK95511_06%26state%3D0%26menuId%3DPAXYK95511_06YZLJ",
                    queryBanner: "abs:http://test-txt.pingan.com.cn:8080/hm_smp_web/creditcardarea/interact/queryBanner.do",

                    creditCardService: {
                        getRefuelDiscountInfo: "rest/refuelDiscount/getRefuelDiscountInfo",
                        checkUser: "rest/seckill/checkUserType",
                        baseURL: "http://test-txt.pingan.com.cn:8080/hm_smp_web/"
                    }
                },
                prd: {
                    creditCardBaseInfo:"http://public-txt.pingan.com.cn/HMP/rest/getExemptToken?appid=10008&url=http%3A%2F%2Feim-txt.pingan.com.cn%2Ftxt%2FxykQueryBusiness%3FweappNo%3DPAXYK95511_06",
                    creditCardBind:"http://public-txt.pingan.com.cn/HMP/rest/getExemptToken?appid=10008&url=http%3A%2F%2Feim-txt.pingan.com.cn%2Ftxt%2FxykFreeLogin%3FweappNo%3DPAXYK95511_06%26state%3D0%26menuId%3DPAXYK95511_06YZLJ",
                    baseURL: buildURL("www.pingan.com", baseRoot),
                    queryBanner: "abs:http://txt.pingan.com.cn/hm_smp_web/creditcardarea/interact/queryBanner.do",

                    creditCardService: {
                        baseURL: "http://txt.pingan.com.cn/hm_smp_web/"
                    }
                }
            };
        })()
    };
    config = window.config = new Config(config)
    var cfg = config.get(),
        _$doc = $(document);
    //检查APP getConfig方法是否能正常调用
    App.IS_TXT ? App.getConfig(function(data) {//获取Native APP环境配置
        data = typeof data == "string" ? JSON.parse(data) : data;
        var section = 'prd', cfg = config.get();
        switch (true) {
            case data["CONFIG_TAG"].indexOf("dev") > -1 :
                section = "dev";
                break;
            case data["CONFIG_TAG"].indexOf("stg") > -1 :
                section = "stg";
                break;
            case data["CONFIG_TAG"].search(/prd|uat/) > -1 :
                section = "prd";
                break;
        }
        config.section(section);
        window.DEBUG = cfg.debug && Boolean(cfg.section.indexOf('prd') == -1); !DEBUG && console.warn('DEBUG is %s', DEBUG);
        _$doc.trigger("configured");
    }): _$doc.trigger("configured");
    /**
     * 下面注册的ajax全局事件用来监视ajax行为
     */

    _$doc.ajaxSend(function(event, jqXHR, ajaxOptions){
        //allow abort for Ajax request during anytime
        $(this).one('abort.ajax', ajaxOptions, function(e){
            console.info('abort request of URL:'+e.data.url);
            //jqXHR.abort('The request has been aborted from URL:'+e.data.url);
            jqXHR.abort();
        });
    });
    _$doc.ajaxSuccess(function(event, XMLHttpRequest, ajaxOptions){
        console.info("status:"+ XMLHttpRequest.status +" statusText:"+ XMLHttpRequest.statusText +" responseText:"+ XMLHttpRequest.responseText + " url:"+ ajaxOptions.url);
    });
    _$doc.ajaxError(function(event, jqXHR, ajaxOptions, thrownError){
        console.error("status:"+ jqXHR.status +" statusText:"+ jqXHR.statusText +" thrown:"+ thrownError + " url:"+ ajaxOptions.url);
    });
    //所有ajax请求响应队列完成后触发
    _$doc.ajaxStop(function(event){
        $(this).off('abort.ajax');//abort.ajax事件的callbacks没有存在的意义。
    });
    $.ajaxSetup({ timeout:cfg.AjaxTimeout });

    /* mobile UI 配置 */
    //jqm配置
    $( document ).on( "mobileinit",  function () {
        console.log("configure jqm");
        //$.mobile.autoInitializePage = false;//禁止自动渲染页面组件
        //$.mobile.allowCrossDomainPages = true;//允许跨域页面载入当前页面dom中
        //$.mobile.ajaxEnabled = false;//禁用jqm的Ajax提交刷新行为
        //$.mobile.linkBindingEnabled = false;//使得jqm对元素自动绑定的click事件无效
        //$.mobile.hashListeningEnabled = false;//禁用jqm hash导航功能
        //$.mobile.pushStateEnabled = false;//禁用pushState功能，pushState是无刷新更改URL，属于HTML5 history对象的API

        // for loading widget
        $.mobile.loader.prototype.options.text = "请等待...";
        $.mobile.loader.prototype.options.textVisible = true;

        var mat = navigator.userAgent.match(/(Android)\s*([\d\-\.]+)/i);
        mat && mat[1] == 'Android' && parseFloat(mat[2]) < 4.4 && ($.mobile.defaultPageTransition = 'none');
        $.mobile.page.prototype.options.domCache = true;
    });
    $( document ).on("pagebeforechange", function(e, data){
        sessionStorage.setItem("direction", data.options.direction||"");
    })
})(jQuery);