/**
 * 序列化表单
 * @returns {JSON}
 */
$.fn.serializeJSON = function(){
    var data = {};
    $.each($(this).serializeArray(), function(idx, obj){
        data[ obj.name ] = obj.value;
    });
    return data;
}
/**
 * load script file 加载js脚本，如果不需要requireJS的话
 * @param url
 * @param options
 * @returns {*}
 */
$.cachedScript = function( url, options ) {
    // Allow user to set any option except for dataType, cache, and url
    options = $.extend( options || {}, {
        dataType: "script",
        cache: true,
        url: url
    });

    // Use $.ajax() since it is more flexible than $.getScript
    // Return the jqXHR object so we can chain callbacks
    return jQuery.ajax( options );
};
/**
 * 单纯的占位符替换 数据{name: "value"} 模版{{name}} 输出value
 * @param {String} tpl
 * @param {JSON} data
 * @returns {String}
 */
function template(tpl, data){
    $.each(data, function(i, e){
        tpl = tpl.replace(RegExp("{{"+ i +"}}","g"), e);
    })
    return tpl;
}
/**
 * http错误处理，返回实例化后的Error对象
 * @param {jqXHR|XMLHttpRequest} jqXHR
 * @returns {Error}
 */
function httpErrorHandle(jqXHR){
    var status = jqXHR.status,
        statusText = jqXHR.statusText,
        msg;

    switch ( true ) {
        case status == 408 || statusText == "timeout":
            msg = "网络超时";
            break
        case 500 < status && status < 505:
            msg = "服务器错误";
            break;
        case statusText == "error":
            msg = "无网络";
            break
        default:
            msg = "HTTP请求："+jqXHR.statusText;
            break;
    }
    return new Error(msg);
}

//事件模型
var Events = {
    _events: null,
    _eveSplit: /\s+/,
    _nsSplit: /[.:]/,
    _isMatchEve: function(eve, match){
        match = this._split(match, this._nsSplit);
        eve = this._split(eve, this._nsSplit);

        if ( eve[0] && eve[1] ) {//如果事件名称和命名空间都提供了
            return match[0] == eve[0] && match[1] == eve[1];

        } else if ( eve[0] ) {//只提供了事件名称
            return match[0] == eve[0];

        } else if ( eve[1] ) {//只提供了命名空间
            return match[1] == eve[1];
        }
        return false;
    },
    _split: function(name, regex){
        return name.trim().split(regex)
    },
    on: function(name, callback, context){
        this._events || (this._events = {});

        var arrName = this._split(name, this._eveSplit);
        arrName.forEach(function(eve){
            var events = this._events[eve] || (this._events[eve] = []);
            events.push({callback: callback, ctx: context || this});
        },this);
        return this;
    },
    trigger: function (name) {
        if (!this._events) return this;
        var events = [], args = [].slice.call(arguments, 1);

        this._split(name, this._eveSplit).forEach(function(eve){
            for(var key in this._events) {
                this._isMatchEve( eve, key ) && (events = events.concat(this._events[key]));
            }
        }, this);

        events.forEach(function(e){
            e.callback.apply(e.ctx, args)
        });
        return this;
    },
    once: function(name, callback, context) {
        var self = this,
            once = function() {
                self.off(name, once);
                callback.apply(this, arguments);
            };
        this._split(name, this._eveSplit).forEach(function(eve){
            this.on(eve, once, context);
        },this);
        return this;
    },
    off: function(name, callback, context){
        if (!this._events) return this;

        if (!name && !callback && !context) {
            this._events = {};
            return this;
        }

        typeof callback == "object" && (context=callback);

        var self = this,
            args = arguments,
            key,
            remove = function(e, k, events){
                var or = args.length == 2 && (e.callback === callback || e.ctx === context),
                    and = args.length == 3 && (e.callback === callback && e.ctx === context);

                ( or || and ) && events.splice( k, 1 );
                return events.length == 0;
            };

        this._split(name, this._eveSplit).forEach(function(eve){
            for(key in this._events) {
                if ( !this._isMatchEve( eve, key ) ) continue;

                args.length == 1 ?
                    delete self._events[key] :
                    self._events[key].some(remove) && delete this._events[key];
            }
        },this);
        return this;
    }
};

/**
 * 百度经纬度转换成国测局经纬度
 * @param lng 百度经度
 * @param lat 百度纬度
 * @returns {{lat: number, lng: number}}
 */

function bdGPS2gcjGPS(lng, lat){
    var x_pi = 3.14159265358979324 * 3000.0 / 180.0;
    var x = lng - 0.0065, y = lat - 0.006,
        z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * x_pi);
    var theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * x_pi);

    lng = z * Math.cos(theta);
    lat = z * Math.sin(theta);

    return {lat: lat, lng: lng};
}

function gcjGPS2bdGPS(lng, lat){
    var x_pi = 3.14159265358979324 * 3000.0 / 180.0;
    var x = lng, y = lat,
        z = Math.sqrt(x * x + y * y) + 0.00002 * Math.sin(y * x_pi);
    var theta = Math.atan2(y, x) + 0.000003 * Math.cos(x * x_pi);

    lng = z * Math.cos(theta) + 0.0065;
    lat = z * Math.sin(theta) + 0.006;

    return {lat: lat, lng: lng};
}