/**
 * 公共部分的验证规则
 * @require jQuery validate
 List of built-in Validation methods

 A set of standard validation methods is provided:

 required – Makes the element required.
 remote – Requests a resource to check the element for validity.
 minlength – Makes the element require a given minimum length.
 maxlength – Makes the element require a given maxmimum length.
 rangelength – Makes the element require a given value range.
 min – Makes the element require a given minimum.
 max – Makes the element require a given maximum.
 range – Makes the element require a given value range.
 email – Makes the element require a valid email
 url – Makes the element require a valid url
 date – Makes the element require a date.
 dateISO – Makes the element require a ISO date.
 number – Makes the element require a decimal number.
 digits – Makes the element require digits only.
 creditcard – Makes the element require a creditcard number.
 equalTo – Requires the element to be the same as another one
 */
define(['jq/jquery.validate', 'helper/xdate'], function(){
    function toDateString(string){
        var regex = /^(\d{4})(\d{1,2})(\d{1,2})$/,
            matches = string.match(regex);
        return $.isArray(matches) ? matches.slice(1).join('-') : '';
    }

    jQuery.validator.addMethod("telephone", function(tel, element) {
        //test for 0000-00000000
        return this.optional(element) || /^(?:\d{2,4})[\-\s](\d{3,4}){2}$/.test(tel);
    }, "请输入正确的电话号码，格式类似0000-000000");

    jQuery.validator.addMethod("mobile", function(mobile, element) {
        //test for +00 000 0000 0000
        return this.optional(element) || /^(\d{3})(?:[\s\-]*\d{4}){2}$/.test(mobile);
    }, "请输入正确的手机号码");

    jQuery.validator.addMethod("orgPhone", function(tel, element) {
        //test for 0000-00000000-000...
        return this.optional(element) || /^(\d{2,4})([\-\s])(\d{6,8})(\2\d+)?$/.test(tel);
    }, "请输入正确的电话号码，格式类似0000-000000[-000]，方括号内的部分可选");

    jQuery.validator.addMethod("zipcode", function(zipcode, element) {
        //test for +00 000 0000 0000
        return this.optional(element) || /^\d{5,6}$/.test(zipcode);
    }, "请输入正确邮政编码");

    jQuery.validator.addMethod("IDCardCN", function(IDCardCN, element) {
        //test for 000000 00000000 0000
        return this.optional(element) || /^\d{6}\d{4}\d{2}\d{2}(\d{3}[\dx])$/i.test(IDCardCN);
    }, "请输入正确身份证号码");
    jQuery.validator.addMethod("IDCardCN_Birth", function(IDCardCN, element) {
        var matches = IDCardCN.match(/^\d{6}(\d{4})(\d{2})(\d{2})\d{3}[\dx]$/i),
            y = 0, m = 0, d = 0,
            birthday;

        if ($.isArray(matches) ) {
            y = matches[1];
            m = matches[2];
            d = matches[3];
        }
        birthday = new XDate( $.validator.format("{0}-{1}-{2}", y, m ,d) );
        return this.optional(element) || XDate.checkdate(m, d, y) && birthday.diffYears( new XDate ) > 0;
    }, function(params, element){
        var matches = $(element).val().match(/^\d{6}(\d{4}\d{2}\d{2})\d{3}[\dx]$/i);
        return "身份证号码的生日（"+ matches[1] +"）部分有误且不可大于当前日期";
    });
    jQuery.validator.addMethod("isAdult", function(datestring, element, params) {
        var date;
        datestring = toDateString(datestring) || toDateString(params);
        date = new XDate(datestring);
        return this.optional(element) || (date.valid() ? date.diffYears( new XDate ) >= 18 : false);
    }, "您未满18岁");

    jQuery.validator.addMethod('isExpire',function(datestring, element){
        return this.optional(element) || XDate( toDateString(datestring) ).diffSeconds( new XDate ) < 0;
    }, '有效期不可低于当前日期');

    jQuery.validator.addMethod('length',function(length, element, params){
        return this.optional(element) || length.length == params;
    }, jQuery.validator.format('输入长度必须是{0}位'));
    jQuery.validator.addMethod("date", function( value, element ) {
        var dateString = toDateString(value).split("-");
        return this.optional(element) || XDate.checkdate(dateString[1], dateString[2], dateString[0]);
    }, "请输入合法的日期");

    //css绑定复合规则
    /*
    //中文姓名
    $.validator.addClassRules("name",{
        required: true,
        chinese:true,
        maxlength:30
    });
    //身份证号码的校验
    $.validator.addClassRules("IDCard", {
        length: 18,
        IDCardCN: true,
        IDCardCN_Birth: true
    });*/
});
