/**
 * 定义最外层的视图对象
 */
define(["commonView/waitingView"], function(WaitingView) {
    var loop=function(){};
    var WrapView = PAWA.PAWAView.extend({
        // 标签id
        id : 'wrapView',
        name : 'wrapView',
        tagName : 'div',
        // 顶部视图对象
        headerView : null,
        // 内容区域视图对象
        bodyView : null,
        // 底部视图对象
        footerView : null,
        // 渲染视图对象
        render : function() {
            var self = this;

            // 将当前对象的DOM设置到<body>区域
            $('body').append(this.el);
            self.bgLayer = $(".bg-model");
        },
        // 设置顶部视图对象
        // @param {Object} ViewClass
        // @param {Object} options
        setHeader : function(ViewClass, options) {
            // 如果当前已经存在顶部视图, 且与需要设置的为同一对象, 则忽略
            var headerView = this.headerView;
            if(headerView && this.headerView instanceof ViewClass) {
                return headerView;
            } else if(headerView) {
                // 目前已存在顶部视图, 但新的视图对象与当前视图不一样, 则销毁后重建
                headerView.destroy();
            }

            // 重建顶部视图对象
            headerView = new ViewClass(options);
            // 将视图对象设置到外层视图的最前面
            this.$el.prepend(headerView.el);

            this.headerView = headerView;
            return headerView;
        },
        // 设置内容区域视图
        // @param {Object} ViewClass
        // @param {Object} options
        setBody : function(ViewClass, options) {
            // 与顶部视图对象的设置流程一致
            var bodyView = this.bodyView;
            if(bodyView && this.bodyView instanceof ViewClass) {
                return bodyView;
            } else if(bodyView) {
                bodyView.destroy();
            }
            bodyView = new ViewClass(options);
            // 内容区域视图将会被添加到顶部视图对象的后面
           if(this.headerView && this.headerView.$el)
                this.headerView.$el.after(bodyView.el);
            else
                this.$el.prepend(bodyView.el);

            this.bodyView = bodyView;
            return bodyView;
        },
        // 设置底部视图
        // @param {Object} ViewClass
        // @param {Object} options
        setFooter : function(ViewClass, options) {
            // 与顶部视图对象的设置流程一致
            var footerView = this.footerView;
            if(footerView && this.footerView instanceof ViewClass) {
                return footerView;
            } else if(footerView) {
                footerView.destroy();
            }
            footerView = new ViewClass(options);
            // 底部视图会被添加到外层视图的最后面
            this.$el.append(footerView.el);

            this.footerView = footerView;
            return footerView;
        } ,
        /**
         *
         * 展示只有一个确认按钮的提示框
         * @param options {info:(提示信息),btnVal:'按钮名称',data:'根据需要输入数据参数',btnCallback:'回调函数'}
         */
        tipsOpened:false,
        showTips:function(options){
            var self = this;
            var opts = _.extend({
                info: "请确认",
                btnVal: "确认",
                data: null,
                btnCallback: loop
            }, options);
            if(self.tipsOpened){
                return ;
            }
            self.tipsOpened = true;
            require(["commonView/TipsView"], function(TipsView) {
                var TipsView = new TipsView(opts);
                wrapView.bgLayer.after(TipsView.el);
            });
        },
        showWaiting:function(options){
            var self = this;
            var opts = _.extend({
                data: null,
                info:'努力加载中...'
            }, options);
            var waitingView = PAWA.PAWAView.shareViews.get("waitingView");
            if(waitingView) {
                waitingView.show(options);
                return ;
            }
            var waitingView = new WaitingView(opts);
            waitingView.show(options);
            self.$("#bg-model").show();
            wrapView.bgLayer.after(waitingView.el);
            PAWA.PAWAView.shareViews.add("waitingView", waitingView);
        },
        hideWaiting: function() {
            var waitingView = PAWA.PAWAView.shareViews.get("waitingView");
            if(waitingView) {
                waitingView.hide();
            }
        }
    });

    return WrapView;
});
