define(['backbone', "native"], function () {
    var Page, Nav, Header, Content;
    Page = Backbone.View.extend({
        tagName: 'div',
        attributes: {
            class: 'page',
            "data-role": "page"
        },
        Header: null,
        Content: null,
        initialize: function (options) {
            var $html = options.$html
            this.options = options;
            this.$el.html($html);
            this.Header = new Header({ $html: $html.filter(".header") }),
            //this.Nav = new Nav({ $html: this.options.nav }),
            this.Content = new Content({ $html: $html.filter(".content") });
        },
        render: function () {
            this.$el.appendTo(document.body);
            this.focusPage();
            return this;
        },
        focusPage: function(){
            this.$el.siblings().removeClass("active");
            this.$el.addClass("active");
            this.trigger("pageshow");
            this.Header.title( this.Header.$el.find("h1:first, .title:first").text() );
            return this;
        },
        fix: function(){
            var shimClassName = "click through shim";
            $(document).one("pageinit", function(){
                if ( $.mobile.pageContainer.children( "." + shimClassName.replace(/\s+/g, ".") ).size() == 0 ) {
                    var $shim = $("<div></div>")
                            .width( $(document).width() )
                            .height( $(document).height()),
                        $page, $autofocus;

                    $shim.addClass(shimClassName);
                    window.DEBUG && $shim.text("穿透事件阻挡层");

                    $.mobile.pageContainer.append( $shim );
                    $(document).off(".showShim").bind("pagebeforechange.showShim", function(){
                        console.debug("show shim to fix click through");
                        $shim.show().delay(350).hide(function(){
                            console.debug("hide shim");
                           /*//simulate autofocus behavior for first element
                            $page = $.mobile.pageContainer.children("[data-role=page]:visible");
                            $autofocus = $page.find("input[autofocus]:visible:first");
                            console.debug($page.find(":focus:visible").size());
                            console.debug($autofocus.size());
                            if ( $page.find(":focus:visible").size() == 0 && $autofocus.size() > 0 ) {
                                console.debug($autofocus.attr("name"));
                                $autofocus.focus();
                                (_.unearth("cordova.exec", window) || $.noop)(function(info){}, function(info){}, 'CommonTools', 'hideOr', []);
                            }*/
                        });
                    });
                }
            });
        },
        /*UI控制部分代码，必要时分离或移除*/
        ui: function(){
            var self = this;
            return {
                popup: function(selector, method){
                    var $popup = self.$( selector),
                        $overlay = $popup.prev(".overlay"),
                        width = $popup.width(),
                        height = $popup.height(),
                        toggle = function(){
                            $popup.toggleClass("ui-popup-hidden")
                                .prev(".overlay").toggleClass("ui-screen-hidden");
                        };
                    $overlay.off("click").bind("click", toggle);
                    switch (method) {
                        case "toggle":
                                toggle();
                            break;
                        case 'reposition':
                                //move to center of the window
                                $popup
                                    .css({
                                        left: ( $(window).width() - width ) / 2,
                                        top: ( $(window).height() - height ) / 2
                                    });
                            break;
                    }
                }
            }
        }
    });
    Nav = Backbone.View.extend({
        events: {
            'touchend [data-hash]': 'navigator'
        },
        initialize: function (options) {
            this.setElement(options.$html);
        },
        navigator: function (e) {
            this.lastTarget && this.lastTarget.removeClass('current');
            this.lastTarget = $(e.currentTarget);
            this.lastTarget.addClass('current');
        },
        link: function () {
            var links = [];
            this.$el.find('[data-hash]').each($.proxy(function (idx, el) {
                links.push($(el).data('hash'));
            }, this));
            return links;
        },
        render: function () {
            this.lastTarget = this.$el.find('[data-hash^="' + location.hash.slice(1) + '"]:first');
            this.lastTarget.addClass('current');
            return this;
        }
    });
    Header = Backbone.View.extend({
        _title: '标题',
        _defaultButton: {
            '.home': function (e) {
                /*e.preventDefault();
                 e.stopPropagation();*/
                //返回应用中心
            }
            /*
             ,'.back': function(e){
             this.Controller.back();
             },
             '.forward': function(e){
             this.Controller.forward();
             },
             '.prev': function(e){},
             '.next': function(e){},
             '.productShow': function(e){},
             '.save': function(e){},
             '.apply': function(e){},
             '.query': function(e){},
             '.task': function(e){},
             '.sendAll': function(e){}*/
        },
        /**
         * 存储动态更新后的当前显示按钮配置
         */
        _currentButton: {},
        events: {
            "tap [data-rel=back]": function(e){
                e.preventDefault();
                window.history.back();
            }
        },
        initialize: function (options) {
            this.setElement(options.$html);
            //this.title(this.$el.find("h1").text());
            //this.button();
        },
        /**
         * 定制标题按钮
         * @param button { selector: callbck }
         */
        button: function (button) {
            var lastAction = Controller.router.getLastAction(),
                hash = lastAction.controller + '/' + lastAction.action;

            if (_.isUndefined(button)) return this._currentButton[ hash ];

            var button = this._currentButton[ hash ] = _.extend({}, this._defaultButton, button),
                selector = '.button',
                buttonEl = this.$el.find(selector);

            //隐藏按钮
            buttonEl.removeClass('dsib').addClass('dsn');
            buttonEl.parent().addClass('dsn');

            //注册事件及显示按钮
            this.$el.off(selector);
            _.each(button, function (val, key) {
                buttonEl = this.$el.find(selector + key);

                if (buttonEl.size() == 0) return true;

                _.isFunction(val) && this.$el.on('click' + selector, selector + key, _.bind(function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                    val.apply(this, arguments);
                }, this));

                if (val == false) {
                    delete button[key];
                    buttonEl.removeClass('dsib').addClass('dsn');
                    buttonEl.siblings(':visible').size() == 0 && buttonEl.parent().addClass('dsn');
                } else {
                    buttonEl.addClass('dsib').removeClass('dsn').parent().removeClass('dsn');
                }

            }, this);
        },
        title: function (text) {
            if (text) {
                this._title = text;
                document.title = text;
                //this.$el.find('.title:first').text(text);
                App.setTitle(text);
                $(document).trigger("titled");
            }
            return this._title;
        },
        /**
         * 在头部抛出一条提示消息
         * @param {string} msg 需要抛出的消息字符串
         */
        message: function (msg) {
            var fxOff = jQuery.fx.off,
                $message = this._$message = this._$message || this.$el.find('.message:first');

            $message = $message.clone();
            this.$el.append($message);

            jQuery.fx.off = false;
            $message
                .queue(function () {
                    $(this).text(msg);
                    var left = ( $(this).parent().width() - $(this).width() ) / 2;
                    $(this).css({
                        left: left
                    }).slideDown('slow').dequeue();
                })
                .delay(1500)
                .queue(function () {
                    $(this).slideUp('slow',function () {
                        $(this).remove();
                    }).dequeue();
                    jQuery.fx.off = fxOff;
                });

            return $message;
        }
    });
    Content = Backbone.View.extend({
        tagName: 'div',
        attributes: {
            class: 'content'
        },
        initialize: function (options) {
            this.setElement(options.$html);
        }
    });
    return Page;
});
