/**
 * 定义PAWA对象
 */
define(['backbone', 'logger'], function(Backbone, logger) {
	// 将PAWA原型引用为Backbone, 便于调用和扩展
	var PAWAConstructor = new Function();
	PAWAConstructor.prototype = Backbone;
	PAWAConstructor.prototype.constructor = PAWAConstructor;

	// 创建并扩展PAWA对象
	_.extend(window.PAWA = new PAWAConstructor(), {
		// 框架版本号
		VERSION : '0.0.4',
		// 上一次自动加载时的Hash
		lastHash : '',
		// 初始化创建Router对象
		/**
		 *
		 * @param {Object} options 定义初始化参数
		 *  {
		 * 	isSPA: 是否是单页, true 单页， 否则是多页 , 默认false
		 * 	pushState: 是否使用history api 默认false
		 *  defaultHash: 如果地址栏上没有带hash的话，默认route到的位置  默认"index/index"
		 * }
		 */
		module : "PAWA",
		init : function(options) {
			var options = options || {};
			options.isSPA === undefined && (options.isSPA = false);
			options.pushState === undefined && (options.pushState = false);
			options.defaultHash === undefined && (options.defaultHash = "index/index");
			this.router = new PAWA.PAWARouter();
			this.history.start({
				pushState : options.pushState
			});
			this.defaultHash = options.defaultHash;
			if (options.isSPA) {
				PAWA.history.handlers.push({
					callback : _.bind(this.autoload, this),
					route : /^(.*)$/
				});
				this.navigate();
			}
			this.handleStatus.coreConfig = options.coreConfig;
		},
		// 导航到当前的action
		navigate : function(fragment) {
			var router = this.router, fragment = fragment || location.hash.substring(1) || this.defaultHash;
			router.navigate('', {
				trigger : false,
				replace : true
			});
			router.navigate(fragment, {
				trigger : true,
				replace : true
			});
		},
		// 自动解析并加载模块入口
		autoload : function(hash) {
			if (this.lastHash === hash) {
				PAWA.routeStatus = 404;
				PAWA.handleStatus(PAWA.routeStatus, hash);
				throw Error('404: not found uri: ' + hash);
				return false;
			}
			this.lastHash = hash;
			var hashArr = hash.split('/');
			var module = hashArr.splice(0, 1), controller = hashArr.join('/');
			if (module && module != 'common') {
				var mainPath = 'app/modules/' + module + '/main';
				require([mainPath], _.bind(function(moduleMain) {
					moduleMain();
                    if(location.hash.substring(1) === hash)
					    this.navigate(hash);
				}, this));
			}
		},
		/**
		 * 给PAWA应用增加过滤器
		 * @param {Object} filterName 过滤器名称
		 * @param {Object} filterObj 过滤器对象
		 * @param {Object} position 添加到过滤器中的位置
		 * @param {Boolean} prevOrNext 添加到position（当且仅当postion是一个string）的前面还是后面，true为前面，false为后面，默认后面
		 */
		addFilter: function addFilter(filterName, filterObj, position, prevOrNext) {
			var filterChain = this.__filterChain__, 			//拿到责任链对象，这里不使用链表，用数组标识顺序
				//拿到责任链标识顺序的数组
				sortedName = filterChain._sortedName,			
				//拿到责任链中的过滤器对象
				filters = filterChain._filters,					
				//查看当前要添加的过滤器在责任链中是否存在
				filterIndex = sortedName.indexOf(filterName),	
				//从尾部增加
				isAddToTail = position === sortedName.length || undefined === position,	
				//从头部增加标识
				isAddToHead = position === 0,					
				//从指定位置增加标识
				isAddAbsolute = typeof position === 'string' && sortedName.indexOf(position);	
			
			if(filterIndex !== -1) {				//如果责任链中已经有当前名称的filterName则报错,并且不再进行处理
				throw new FilterAreadyInChainError(filterName);
				return ;
			}
			if(isAddToTail)	{			//添加到尾部
				sortedName.push(filterName);
				filters[filterName] = filterObj;	
			} else if(isAddToHead) {	//添加到头部
				sortedName.unshift(filterName);
				filters[filterName] = filterObj;
			} else if(isAddAbsolute !== -1) {	//从指定的地方添加
				if(prevOrNext) {	//true为前面添加
					if(isAddAbsolute === 0) {	//如果是在第0个位置，则unshift到前面
						sortedName.unshift(filterName);
					} else {	
						//否则使用splice方法从中间加入
						sortedName.splice(isAddAbsolute-1, 0, filterName);
					}	
				} else {	//默认添加到后面
					sortedName.splice(isAddAbsolute, 0, filterName);
				}
				//将过滤器对象添加到责任链中
				filters[filterName] = filterObj;
			}
			return this;
		},
		/**
		 * 删除一个过滤器
 		 * @param {Object} filterName 过滤器名
		 */
		removeFilter: function removeFilter(filterName) {
			var filterChain = this.__filterChain__, 			//拿到责任链对象，这里不使用链表，用数组标识顺序
			//拿到责任链标识顺序的数组
			sortedName = filterChain._sortedName,
			//拿到责任链中的过滤器对象
			filters = filterChain._filters,					
			//查看当前要添加的过滤器在责任链中是否存在
			filterIndex = sortedName.indexOf(filterName);
			if(filterIndex !== -1) {
				sortedName.splice(filterIndex, 1);
			}
			return this;
		},
		/**
		 * 解析过滤器
		 * @param {Object} routeReg route的正则表示
		 * @param {Object} fragment 当前帧
		 * @param {Object} args 其余的参数信息
		 */
        execFilter: function execFilters(routeReg, fragment, args, filterIndex) {
            var filterChain = this.__filterChain__, 			//拿到责任链对象，这里不使用链表，用数组标识顺序
            //拿到责任链标识顺序的数组
                sortedName = filterChain._sortedName,
            //拿到责任链中的过滤器对象
                filters = filterChain._filters;
            filterIndex = filterIndex || 0;
            //拿到责任链当前的过滤器
            var currFilter = filters[sortedName[filterIndex]];
            if(!currFilter) return ;
            if(typeof currFilter === 'object') {
                for(var i = 0, len = currFilter.conditions.length;i < len; i ++) {
                    if(currFilter.conditions[i].test(fragment)) {
                        //调用责任链的处理方法,filter只调用一次
                        currFilter.handler(routeReg, fragment, args, filterIndex);
                        break;
                    }
                }
                if(i === len) {
                    this.execFilter(routeReg, fragment, args, filterIndex + 1);
                }
            }
            return this;
        },
		OK_STATUS: '200, 304, 201, 204',
		__filterChain__: {
			_sortedName: [],
			_filters: {
			}
		},
		/**
		 * 根据状态码做对应的处理
 		 * @param {Object} statusCode
		 */
		handleStatus: function(statusCode, route, fragment, args) {
			var handlers = this.handleStatus.coreConfig[statusCode], isContinue = true;
			if(!handlers) return;	
			for(var i = 0, len = handlers.length; i < len; i ++) {
				if(typeof handlers[i] === 'function') {
					isContinue = handlers[i](route, fragment, args);
				}	
				if(isContinue === false) return ;
			}		
		}
	});


    // PAWAController
    var PAWAController = PAWA.PAWAController = function(options) {
        this.initialize.apply(this, arguments);
    }
    // Controller继承Backbone.Events, 因此允许使用Events中的事件方法
    _.extend(PAWAController.prototype, Backbone.Events, {
        initialize : function(options) {
            var actions = this.actions;
            if(actions) {
                var router = PAWA.router, list = [], route = null, prefix = this.module + '/' + this.name;
                for(var key in actions) {
                    list.unshift([key, this[actions[key]]]);
                }

                for(var i = 0, len = list.length; i < len; i++) {
                    route = list[i];
                    router.route(prefix + (route[0] === '' ? '' : "/" + route[0]), '', _.bind(route[1], this));
                }
            }
            //向前兼容
            (!!this.__recordLog__) && this.__recordLog__();
            this.init.apply(this, arguments);
        },
        // 模块名
        module : '',
        // 控制器名, 用于模块路由时定义规则
        name : '',
        // 模块规则
        actions : null,
        // 初始化方法
        init : function(options) {
        },
        // 在销毁控制器时同时移除URL中的Action监听
        destroy : function() {
            var actions = this.actions;
            if(actions) {
                var router = PAWA.router, handlers = PAWA.history.handlers, handler = null, prefix = this.module + '/' + this.name + '/';
                for(var key in actions) {
                    var exp = router._routeToRegExp(prefix + key).toString();

                    for(var i = 0, len = handlers.length; i < len; i++) {
                        handler = handlers[i];
                        if(handler.route.toString() == exp) {
                            handlers.splice(i, 1);
                            break;
                        }
                    }
                }
            }
        }
    });

    // 添加控制器的继承方法, 在继承子控制器时自动添加getInstance获取单例的方法
    PAWAController.extend = function(protoProps, classProps) {
        var Controller = PAWA.PAWAModel.extend.call(this, protoProps, classProps);

        Controller.getInstance = function(options) {
            if(!this._instance) {
                this._instance = new this(options);
            }

            return this._instance;
        }
        return Controller;
    }

    // PAWAModel
    var PAWAModel = PAWA.PAWAModel = Backbone.Model.extend({
        initialize : function(options) {
            this.init.apply(this, arguments);
            //向前兼容
            (!!this.__recordLog__) && this.__recordLog__();
        },
        init : function(options) {
            Array.prototype.push.apply(this._fetchInterceptors, this._globalInterceptors);
            Array.prototype.push.apply(this._saveInterceptors, this._globalInterceptors);
        },
        _fetchInterceptors: [],
        _saveInterceptors: [],
        _globalInterceptors: [],
        /**
         * 带有权限检查的查询，如果返回的用户权限不正确，则
         */
        fetch: function(options) {
            var self = this;
            options = options ? _.clone(options) : {};
            if (options.parse === void 0) options.parse = true;
            var success = options.success;
            options.success = function(model, resp, options) {
                //数据转换
                var serverAttrs = model.parse(resp, options);

                var interceptors = self._fetchInterceptors, iLen = interceptors.length, interceptResult;
                while(iLen --) {
                    interceptResult = interceptors[iLen](serverAttrs, model, resp, options);
                    if(interceptResult === false) { //触发已被拦截无需继续执行下面的拦截器
                        return false;
                    }
                }

                if (!model.set(serverAttrs, options)) return false;
                if (success) success(model, resp, options);
            };
            return this.sync('read', this, options);
        },
        save: function(key, val, options) {
            var self = this, attrs, success, method, xhr, attributes = this.attributes;

            // Handle both `"key", value` and `{key: value}` -style arguments.
            if (key == null || typeof key === 'object') {
                attrs = key;
                options = val;
            } else {
                (attrs = {})[key] = val;
            }

            // If we're not waiting and attributes exist, save acts as `set(attr).save(null, opts)`.
            if (attrs && (!options || !options.wait) && !this.set(attrs, options)) return false;

            options = _.extend({validate: true}, options);

            // Do not persist invalid models.
            if (!this._validate(attrs, options)) return false;

            // Set temporary attributes if `{wait: true}`.
            if (attrs && options.wait) {
                this.attributes = _.extend({}, attributes, attrs);
            }

            // After a successful server-side save, the client is (optionally)
            // updated with the server-side state.
            if (options.parse === void 0) options.parse = true;
            success = options.success;
            options.success = function(model, resp, options) {
                // Ensure attributes are restored during synchronous saves.
                model.attributes = attributes;
                var serverAttrs = model.parse(resp, options);
                if (options.wait) serverAttrs = _.extend(attrs || {}, serverAttrs);

                var interceptors = self._saveInterceptors, iLen = interceptors.length, interceptResult;
                while(iLen --) {
                    interceptResult = interceptors[iLen](serverAttrs, model, resp, options);
                    if(interceptResult === false) { //触发已被拦截无需继续执行下面的拦截器
                        return false;
                    }
                }

                if (_.isObject(serverAttrs) && !model.set(serverAttrs, options)) {
                    return false;
                }

                if (success) success(model, resp, options);
            };

            // Finish configuring and sending the Ajax request.
            method = this.isNew() ? 'create' : (options.patch ? 'patch' : 'update');
            if (method === 'patch') options.attrs = attrs;
            xhr = this.sync(method, this, options);

            // Restore attributes.
            if (attrs && options.wait) this.attributes = attributes;

            return xhr;
        }
    });

    // PAWACollection
    var PAWACollection = PAWA.PAWACollection = Backbone.Collection.extend({
        initialize : function(options) {
            this.init.apply(this, arguments);
            //向前兼容
            (!!this.__recordLog__) && this.__recordLog__();
        },
        init : function(options) {
        }
    });


    // PAWAHistory
    var PAWAHistory = PAWA.PAWAHistory = Backbone.History.extend({
        initialize : function(options) {
            this.init.apply(this, arguments);
        },
        init : function(options) {

        }
    });



    // 用于存放应用中共享的模型对象
    PAWAModel.shareModels = {
        // @private
        _shareList : {},
        /**
         * 添加共享的模型
         */
        add : function(name, model) {
            PAWAModel.shareModels._shareList[name] = model;
        },
        /**
         * 获取共享的模型
         */
        get : function(name) {
            return PAWAModel.shareModels._shareList[name];
        },
        /**
         * 移除模型
         */
        remove : function(name) {
            var shareList = PAWAModel.shareModels._shareList;
            shareList[name] = null;
            delete shareList[name];
        }
    }

    // PAWARequest
    var request = PAWA.request = {
        _parameters: {},
        clear : function() {
            if(!_.isEmpty(this._parameters))//如果不是空对象则设置成空对象，可以手动清除
                this._parameters = {};
        },
        /**
         *添加或者获取参数的统一方法
         * @param args:
         * 1、若args[0]为字符串，且 只有一个参数，则取出key为args[0]的参数值
         * 2.若args[0]为字符串，且有两个参数，则设置key为args[0]的参数值为args[1]
         * 3. 若第一个参数(args)是一个Object，则将Object中的所有key和value都这是到参数表中
         * 4. 除第 1 条以外的任意一种情况均返回所有参数
         */
        param : function() {
            var arg = arguments, idx = 0, len = arg.length;
            switch(len) {
                case 1: //一个参数的情况
                    if(typeof arg[0] === 'string') {	//传的是一个字符串
                        return _.clone(this._parameters[arg[0]]);
                    } else if(_.isObject(arg[0])) {
                        _.extend(this._parameters, arg[0]);
                    }
                    break;
                case 2: //两个参数的情况
                    if(typeof arg[0] === 'string') {	//传的是一个字符串
                        return this._parameters[arg[0]] = arg[1];
                    } else if(_.isObject(arg[0])) {
                        _.extend(this._parameters, arg[0]);
                    }
                    break;
            }
            return _.clone(this._parameters);
        }
    };

    // PAWARouter
    var PAWARouter = PAWA.PAWARouter = Backbone.Router.extend({
        initialize : function(options) {
            this.init.apply(this, arguments);
        },
        init : function(options) {
        },
        route: function(route, name, callback) {

            if (!_.isRegExp(route)) route = this._routeToRegExp(route);
            if (!callback) callback = this[name];
            var statusReg = new RegExp();
            Backbone.history.route(route, _.bind(function(fragment) {
                //如果当前帧已经改变，不再触发当前帧的后续动作 (包括过滤器和控制器)
                if(location.hash.substring(1) !== fragment) return this;

                var args = this._extractParameters(route, fragment);
                PAWA.routeStatus = 200;
                PAWA.execFilter(route, fragment, args);
                statusReg.compile(PAWA.routeStatus, 'ig');
                //判断经过过滤器之后状态码是否依旧OK，如果不OK的话，执行配置中的状态码处理
                if(!statusReg.test(PAWA.OK_STATUS)) {
                    PAWA.handleStatus(PAWA.routeStatus, route, fragment, args);
                    return ;
                }

                //状态是OK的，继续其它处理
                callback && callback.apply(this, args);
                this.trigger.apply(this, ['route:' + name].concat(args));
                this.trigger('route', name, args);
                Backbone.history.trigger('route', this, name, args);
                PAWA.request.clear();
            }, this));
            return this;
        }
    });
    PAWARouter.routeStatus = 200;


    // PAWAView
    var PAWAView = PAWA.PAWAView = Backbone.View.extend({
        // 用于视图关联时的标识
        name : '',
        // 视图所引用的Controller
        controller : null,
        initialize : function(options) {
            PAWAView.__super__.initialize.apply(this, arguments);
            if(options) {
                this.controller = options.controller;
                this.top = options.top;			//最顶层的视图对象
                this.parent = options.parent;	//父层的视图对象
            }
            // 存放所有子视图对象
            this.children = [],
                // 根据名称存放所有子视图对象
                this.childrenAsName = {},
                // 根据表达式存放所有子视图列表
                this.childrenAsEl = {},
                // 等待加入到视图中的子视图对象列表
                this.awaitChildren = [],
                //向前兼容
                (!!this.__recordLog__) && this.__recordLog__();
            this.init.apply(this, arguments);
        },
        init : function(options) {
        },
        // 显示视图对象
        show : function() {
            this.$el.show();
        },
        // 隐藏视图对象
        hide : function() {
            this.$el.hide();
        },
        /**
         * 添加一个子视图对象到当前视图中
         * @param {String} el 添加到视图的节点表达式
         * @param {String} name 子视图名称
         * @param {View Object} view 子视图对象
         * @param {Boolean} await 当节点不存在时是否将子视图加入到待渲染列表
         */
        addChild : function(el, name, view, await) {
            // 子视图添加的节点
            var $node = this.$el;
            if(el) {
                $node = $node.find(el);
            }

            if($node.length) {
                // 节点存在, 立即添加到视图
                $node.append(view.el);
                view.parent = this;

                this.children.push({
                    el : el,
                    name : name,
                    view : view
                });
                this.childrenAsName[name] = view;

                if(!this.childrenAsEl[el]) {
                    this.childrenAsEl[el] = [view];
                } else {
                    this.childrenAsEl[el].push(view);
                }
                if(!view.parent) view.parent = this;	//给子视图添加父层视图引用
                if(!view.controller) view.controller = this.controller;	//给子视图添加控制器引用
            } else if(await || await === undefined) {
                // 节点不存在, 存储在待添加列表
                this.awaitChildren.push({
                    el : el,
                    name : name,
                    view : view
                });
            }
            return this;
        },
        /**
         * 渲染所有待添加的子视图列表
         */
        renderChildren : function() {
            var awaitChildren = this.awaitChildren, child = null;

            for(var i = 0, len = awaitChildren.length; i < len; i++) {
                child = awaitChildren[i];
                this.addChild(child.el, child.name, child.view, false);
            }
        },
        /**
         * 清除所有的待添加子视图列表
         */
        clearAwaitChildren : function() {
            this.awaitChildren.length = 0;
        },
        /**
         * 根据视图名称获取视图对象
         */
        getChildByName : function(name) {
            return this.childrenAsName[name];
        },
        /**
         * 根据表达式获取视图对象
         */
        getChildsByEl : function(el) {
            return this.childrenAsEl[el];
        },
        /**
         * 获取所有子视图列表
         */
        getChildren : function() {
            return this.children;
        },
        // @private
        _destroyChild : function(type, value) {
            var children = this.children, child = null, childName = '', childEl = '', childValue = '';

            for(var i = 0, len = children.length; i < len; i++) {
                child = children[i];
                childName = child.name;
                childEl = child.el;
                childValue = child[type];

                if(childValue == value) {
                    child.view.destroy();

                    this.childrenAsName[childName] = null;
                    this.childrenAsEl[childEl] = null;
                    delete this.childrenAsName[childName];
                    delete this.childrenAsEl[childEl];
                    break;
                }
            }
        },
        /**
         * 根据名称销毁子视图
         * @param {String} name
         */
        destroyChildByName : function(name) {
            this._destroyChild('name', name);
        },
        /**
         * 根据表达式销毁子视图
         * @param {String} el
         */
        destroyChildByEl : function(el) {
            this._destroyChild('el', el);
        },
        /**
         * 销毁所有子视图对象
         */
        destroyChilds : function() {
            var children = this.children;
            for(var i = children.length - 1; i >= 0; i--) {
                child = children[i];
                children.splice(i, 1);
                child.view.destroy();
            }

            this.clearAwaitChildren();
            this.childrenAsName = {};
            this.childrenAsEl = {};
            this.children.length = 0;
        },
        remove: function() {
            PAWAView.__super__.remove.apply(this, arguments);
            // 销毁子视图对象
            this.destroyChilds();
            // 解除事件绑定
            this.undelegateEvents();
            this.parent = null;
            this.controller = null;
            this.top = null;
            this.model = null;
        },
        /**
         * 销毁当前视图对象, 同时将销毁所有子视图
         */
        destroy : function() {
            this.remove();
        }
    });

    // 用于存放应用中共享的视图对象
    PAWAView.shareViews = {
        // @private
        _shareList : {},
        /**
         * 添加共享的视图
         */
        add : function(name, view) {
            this._shareList[name] = view;
        },
        /**
         * 获取共享的视图
         */
        get : function(name) {
            return this._shareList[name];
        },
        /**
         * 移除视图
         */
        remove : function(name) {
            var shareList = this._shareList;
            shareList[name] = null;
            delete shareList[name];
        }
    }


    // PAWALoggerAdapter
    function recordLog (){
        var methods = this.constructor.prototype, recorded = this.constructor.__recorded__,
            forbids = {'initialize': 0, 'model':0, 'constructor': 0};
        //因为是在类实例化的时候去做切面日志，因此如果已经实例化过一次的类，就不再做切面日志了
        if(this.constructor.__recorded__) return ;
        //标识为日志已做
        this.constructor.__recorded__ = true;
        for(var name in methods) {
            if(methods.hasOwnProperty(name) && typeof methods[name] === 'function'  && !(name in forbids) && name.charAt(0) !== '_') {
                var method = methods[name];
                methods[name] = (function(name, method) {
                    return function(){
                        var result;
                        if(this.__logCnf__) {
                            logger.i(this.__logCnf__.moduleName, "类：", this.__logCnf__.className, "的方法:",name,"调用开始！");

                            result = method.apply(this, arguments);

                            logger.i(this.__logCnf__.moduleName, "类：",this.__logCnf__.className, "的方法:",name,"调用结束！");
                        } else {
                            result = method.apply(this, arguments);
                        }
                        return result;
                    }
                })(name, method);
            }
        }
    }
    /**
     *配置各个类的切面日志 模块及类名
     */
    PAWA.PAWAController.prototype.__logCnf__ = {moduleName: "PAWA", className: "PAWAController"};
    PAWA.PAWACollection.prototype.__logCnf__ = {moduleName: "PAWA", className: "PAWACollection"};
    PAWA.PAWAModel.prototype.__logCnf__ = {moduleName: "PAWA", className: "PAWAModel"};
    PAWA.PAWAView.prototype.__logCnf__ = {moduleName: "PAWA", className: "PAWAView"};
    /**
     *包装切面日志
     */
    PAWA.PAWAController.prototype.__recordLog__
        = PAWA.PAWACollection.prototype.__recordLog__
        = PAWA.PAWAModel.prototype.__recordLog__
        = PAWA.PAWAView.prototype.__recordLog__
        = recordLog;



    function FilterAreadyInChainError(filterName) {
		var name = "FilterAreadyInChainError",
		  	message = "filter 添加失败，" + filterName + " 已经存在，请先remove之后再进行添加！";
		var err = new Error(message);
		err.name = name;
		err.type = 'PAWA_FILTER_ERROR';
		err.level = '中';
		return err;
	}
	
	return PAWA;
});
