/**
 * 重载Backbone.Model
 */
define(['backbone'], function() {
    var PAWAModel = PAWA.PAWAModel = Backbone.Model.extend({
        initialize : function(options) {
            this.init.apply(this, arguments);
			//向前兼容
            (!!this.__recordLog__) && this.__recordLog__();
        },
        init : function(options) {
            Array.prototype.push.apply(this._fetchInterceptors, this._globalInterceptors);
            Array.prototype.push.apply(this._saveInterceptors, this._globalInterceptors);
        },
        _fetchInterceptors: [],
        _saveInterceptors: [],
        _globalInterceptors: [],
        /**
         * 带有权限检查的查询，如果返回的用户权限不正确，则
         */
        fetch: function(options) {
            var self = this;
            options = options ? _.clone(options) : {};
            if (options.parse === void 0) options.parse = true;
            var success = options.success;
            options.success = function(model, resp, options) {
                //数据转换
                var serverAttrs = model.parse(resp, options);

                var interceptors = self._fetchInterceptors, iLen = interceptors.length, interceptResult;
                while(iLen --) {
                    interceptResult = interceptors[iLen](serverAttrs, model, resp, options);
                    if(interceptResult === false) { //触发已被拦截无需继续执行下面的拦截器
                        return false;
                    }
                }

                if (!model.set(serverAttrs, options)) return false;
                if (success) success(model, resp, options);
            };
            return this.sync('read', this, options);
        },
        save: function(key, val, options) {
            var self = this, attrs, success, method, xhr, attributes = this.attributes;

            // Handle both `"key", value` and `{key: value}` -style arguments.
            if (key == null || typeof key === 'object') {
                attrs = key;
                options = val;
            } else {
                (attrs = {})[key] = val;
            }

            // If we're not waiting and attributes exist, save acts as `set(attr).save(null, opts)`.
            if (attrs && (!options || !options.wait) && !this.set(attrs, options)) return false;

            options = _.extend({validate: true}, options);

            // Do not persist invalid models.
            if (!this._validate(attrs, options)) return false;

            // Set temporary attributes if `{wait: true}`.
            if (attrs && options.wait) {
                this.attributes = _.extend({}, attributes, attrs);
            }

            // After a successful server-side save, the client is (optionally)
            // updated with the server-side state.
            if (options.parse === void 0) options.parse = true;
            success = options.success;
            options.success = function(model, resp, options) {
                // Ensure attributes are restored during synchronous saves.
                model.attributes = attributes;
                var serverAttrs = model.parse(resp, options);
                if (options.wait) serverAttrs = _.extend(attrs || {}, serverAttrs);

                var interceptors = self._saveInterceptors, iLen = interceptors.length, interceptResult;
                while(iLen --) {
                    interceptResult = interceptors[iLen](serverAttrs, model, resp, options);
                    if(interceptResult === false) { //触发已被拦截无需继续执行下面的拦截器
                        return false;
                    }
                }

                if (_.isObject(serverAttrs) && !model.set(serverAttrs, options)) {
                    return false;
                }

                if (success) success(model, resp, options);
            };

            // Finish configuring and sending the Ajax request.
            method = this.isNew() ? 'create' : (options.patch ? 'patch' : 'update');
            if (method === 'patch') options.attrs = attrs;
            xhr = this.sync(method, this, options);

            // Restore attributes.
            if (attrs && options.wait) this.attributes = attributes;

            return xhr;
        }
    });

    // 用于存放应用中共享的模型对象
    PAWAModel.shareModels = {
        // @private
        _shareList : {},
        /**
         * 添加共享的模型
         */
        add : function(name, model) {
            PAWAModel.shareModels._shareList[name] = model;
        },
        /**
         * 获取共享的模型
         */
        get : function(name) {
            return PAWAModel.shareModels._shareList[name];
        },
        /**
         * 移除模型
         */
        remove : function(name) {
            var shareList = PAWAModel.shareModels._shareList;
            shareList[name] = null;
            delete shareList[name];
        }
    }

    return PAWAModel;
});
