/**
 * Extend for jQuery & underscore & backbone
 * 基础库扩展
 * User: 欣苇
 * Date: 13-5-13
 * Time: 上午11:13
 */
define(['backbone'], function(){
    //重写backbone where方法，支持模糊匹配
    function where(obj, attrs, first) {
        var iterator = function (model) {
            for (var key in attrs) {
                //字符串类型支持%模糊匹配，%str% %str str%
                var val = model instanceof Backbone.Model ? model.get(key) : model[key],
                    lftLike = _.isString(attrs[key]) && attrs[key].substr(0, 1) == '%',
                    rgtLike = _.isString(attrs[key]) && attrs[key].substr(-1) == '%';
                if (lftLike && rgtLike) {
                    return val.indexOf(attrs[key].slice(1, -1)) > -1;
                } else if (lftLike) {
                    var str = attrs[key].slice(1);
                    return val.substr(-str.length) == str;
                } else if (rgtLike) {
                    var str = attrs[key].slice(0, -1);
                    return val.substr(0, str.length) == str;
                } else {
                    if (attrs[key] !== val) return false;
                }
            }
            return true;
        }, args;

        if (arguments.length == 1 || (arguments.length == 2 && _.isBoolean(arguments[1])) ) {
            first = attrs;
            attrs = obj;
            args = [iterator];
        } else {
            args = [obj, iterator];
        }
        if (_.isEmpty(attrs)) return first ? void 0 : [];

        return (args.length == 1 ? this : _)[first ? 'find' : 'filter'].apply(this, args);
    };

    _.extend = $.extend;
    /**
     * 给个对象属性路径，逐级访问属性值
     * @example
     * var obj = {
     *     prop:{
     *         child: function(){},
     *         brother: "hello",
     *         sister: false
     *     }
     * }
     * _.unearth('obj.prop.child')
     * => function (){}
     * _.unearth('obj.prop.brother')
     * => "hello"
     * _.unearth('obj.prop.sister')
     * => false
     * _.unearth('obj.prop.other')
     * => undefined
     * @param path like obj.prop.child
     * @returns {*}
     */
    _.mixin({
        unearth : function (path, context){
            path = String(path).split('.');
            var memory, key, idx = 0;
            while ( key = path.shift() ) {
                if ( !idx ) {
                    memory = (context || window)[ key ];
                } else {
                    memory = _.isString(key) && _.isObject(memory) && _.has(memory, key) ? memory[key] : memory;
                    if ( _.isEmpty(memory) ) break;
                }
                idx++;
            }
            return memory;
        },
        where: where
    });

    //重置表单
    $.fn.resetForm = function(){
        var $elements = $(this);
        $elements.each(function(){
            this.reset();
        });
        return $elements;
    };
    //将表单值序列化成JSON
    $.fn.serializeJSON = function(){
        var data = {};
        $.each($(this).serializeArray(), function(idx, obj){
            data[ obj.name ] = obj.value;
        });
        return data;
    }

    //重写Backbone事件分派逻辑，其中只重构了eventName，selector变量内容，其它不变。
    Backbone.View.prototype.delegateEvents = function (events) {
        if (!(events || (events = _.result(this, 'events')))) return this;
        this.undelegateEvents();
        for (var key in events) {
            var method = events[key];
            if (!_.isFunction(method)) method = this[events[key]];
            if (!method) continue;

            // 增加支持以逗号分隔方式批量绑定事件，例如 'click, mouseover, mousedown .selector1, .selector2': 'callback'
            var matches = key.match(/^\w+(,\s*\w+)*/),
                _events = matches && matches[0] || '',
                selector = key.replace(_events, '').trim(),
                postfix = '.delegateEvents' + this.cid,
                eventName = _events.replace(/[,\s]+/g, postfix + ' ') + postfix;

            method = _.bind(method, this);
            if (selector === '') {
                this.$el.on(eventName, method);
            } else {
                this.$el.on(eventName, selector, method);
            }
        }

        //将业务模型事件映射至视图事件处理器
        _.each(this.bizEvents, function(event){
            this.model && this.collection && console.warn("实例化视图对象，同时传入模型和集合对象可能会" +
                "影响bizEvents（业务模型事件列表）与视图对象的事件处理器绑定至错误的model或collection对象。")

            event = event.trim().split(/\s+/);
            var suffix = event[0].search(/^once/i) == 0 ? "Once" : "";
            event = suffix.length == 0 ? event[0] : event[1];

            this["listenTo"+suffix]( this.model || this.collection , event, function(){
                if ( !_.isFunction(this[event]) ) this[event] = function(){};
                this[event].apply(this, arguments);
            });
        }, this);
    };

    /**
     * filling the form
     * @param {Backbone.Model} Model
     * @param {bool} reset
     * @returns {undefined|object}
     */
    Backbone.View.prototype.fillFields = function(data){
        _(data).each(function(value, name){
            var $node = this.$("[name="+ name +"]"),
                type = $node.prop('type'),
                nodeName = $node.prop('nodeName'),
                oldVal,
                refresh;

            if ( !$node.size() ) return true;//like continue

            switch (true) {
                case /^(?:checkbox|radio)$/i.test( type ):
                    oldVal = $node.filter(':enabled:checked').val();

                    refresh = $node.checkboxradio || $.noop;

                    type.search(/radio/i) != -1
                        && $node.prop('checked', false)
                        && $node.data("role") != "none" && refresh('refresh');

                    $node.filter("[value=\""+ value +"\"]").prop('checked', true)
                        && $node.data("role") != "none" && refresh('refresh');

                    $node = $node.filter(':enabled:checked');//new Element by checked
                    break;
                case /^(?:select)/i.test( nodeName ):
                    oldVal = $node.filter(':enabled:selected').val();

                    refresh = $node.selectmenu || $.noop;
                    $node.children("[value=\""+ value +"\"]").prop('selected', true);
                    $node.data("role") != "none" && refresh('refresh');

                    break;
                case /^(?:input|textarea|keygen)$/i.test( nodeName ):
                    oldVal = $node.filter(':enabled').val();
                    $node.val(value);
                    break;
                default:
                    return true;
                    break;
            }
            _.isUndefined(oldVal) && (oldVal = "");
            $node.val() != oldVal && $node.trigger( "change" );
        });

        return this;
    }

    Backbone.Model.prototype.unset = function (attr, options) {
        var attrs = {};
        _.isArray(attr) && _.each(attr, function (val, key) {
            attrs[val] = void 0;
        });
        _.isString(attr) && (attrs[attr] = void 0);
        return this.set(attrs, _.extend({}, options, {unset: true}));
    };

    Backbone.Collection.prototype.where = where;

    /**
     * 管理错误字段信息
     * @param {string|object} field 可选，字段或字段键值对
     * @param {string} err 可选，错误信息
     * @return {string|object} err|error { field: error }
     */
    Backbone.Model.prototype.error = function (field, err, options) {
        if (typeof field === 'object') {
            options = _.clone(err);
            err = void 0;
        }
        options || (options = {});
        var NS = '_error';
        !this.state(NS) && this.state(NS, {}, {silent: true});
        var error = _.clone(this.state(NS));
        if (_.isString(field)) {
            if (_.isUndefined(err)) return error[field];
            error[field] = err;
        } else if (_.isArray(field)) {
            var result = {};
            _.each(field, function (val) {
                _.isString(val) && ( result[val] = error[val] );
            }, this);
            return result;
        } else if (_.isObject(field)) {
            _.extend(error, field);
        }
        this.state(NS, error, options);
        return error;
    };
    /**
     * 移除错误字段
     * @param {string|array} [field] 字段名称
     * @returns {object}
     */
    Backbone.Model.prototype.removeError = function (field, options) {
        options || (options = {});
        var NS = '_error',
            error = _.clone(this.state(NS)) || {};
        if (field == void 0) {
            return this.removeState(NS, options);
        } else if (_.isArray(field)) {
            for (idx in field) delete error[ field[idx] ];
        } else {
            field != void 0 && delete error[field];
        }
        return this.state(NS, error, options);
    };
    /**
     * 设置数据状态信息
     * @param {string|object} [key] 状态名
     * @param {*} [val] 状态值
     * @return {object} 返回状态值或者所有状态键值对
     */
    Backbone.Model.prototype.state = function (key, val, options) {
        typeof key === 'object' && (options = val);
        options || (options = {});
        var NS = '_state';
        !this.has(NS) && this.set(NS, {}, {silent: true});
        var state = _.clone(this.get(NS)),
            attrs = {};
        if (_.isString(key)) {
            if (_.isUndefined(val)) return state[key];
            state[key] = val;
            attrs[ NS ] = state;
            attrs[ NS + ':' + key ] = val;

        } else if (_.isArray(key)) {
            var result = {};
            _.each(key, function (val) {
                _.isString(val) && ( result[val] = state[val] );
            }, this);
            return result;

        } else if (_.isObject(key)) {
            _.each(key, function (val, attr) {
                state[attr] = val;
                val != void 0 && (attrs[ NS + ':' + attr ] = val);
            }, this);
            attrs[ NS ] = state;
        }
        !_.isEmpty(attrs) && this.set(attrs, options);//allow outside bind event to this attribute
        return this;
    };
    /**
     * 移除数据状态
     * @param {string|array} [key]
     * @returns {object}
     */
    Backbone.Model.prototype.removeState = function (key, options) {
        options || (options = {});
        var NS = '_state';
        var state = _.clone(this.get(NS)) || {},
            attrs = {};
        if (key == void 0) {
            for (key in state) attrs[ NS + ':' + key ] = void 0;
            attrs[ NS ] = void 0;

        } else if (_.isArray(key)) {
            for (idx in key) {
                var attr = key[idx];
                delete state[attr];
                attr != void 0 && (attrs[ NS + ':' + attr ] = void 0);
            }

        } else if (_.isString(key)) {
            delete state[key];
            attrs[ NS + ':' + key ] = void 0;
        }

        !_.isEqual(this.get(NS), state) && this.set(NS, state, options);
        !_.isEmpty(attrs) && this.set(attrs, _.extend({}, options, {unset: true}));
        return this;
    };
    //支持存储空间切换
    Backbone.Model.prototype.storage = Backbone.Collection.prototype.storage = function (name) {
        var store = this.localStorage || _.unearth('collection.localStorage', this);

        if (_.isUndefined(name)) {
            return store;
        } else if (_.isString(name)) {
            if( store instanceof Backbone.LocalStorage ) {
                if (store && store.name == name) return store.NS(name);
                //delete old data 非新的模型
                if (store && this.isNew && !this.isNew()) this.destroy();
            }
            return this.localStorage = new Backbone.LocalStorage(name);
        }
    };
});