/**
 * Created with JetBrains WebStorm.
 * User: 欣苇
 * Date: 13-3-19
 * Time: 下午4:08
 * To change this template use File | Settings | File Templates.
 */
define(['helper/Storage'], function (Storage) {
    /**
     * 对象注册表（或称对象仓库）是一个用于在整个应用空间（application space）内存储对象和值的容器。
     * 通过把对象存储在其中，我们可以在整个项目的任何地方使用同一个对象。这种机制相当于一种全局存储。
     */
    function Registry(namespace){
        this._registry = new Storage(namespace||"Registry", 'session');
        return this;
    }
    _.extend( Registry.prototype, {
        NS: function(namespace){
            this._registry.NS(namespace);
            return this;
        },
        reg: function (key, val) {
            _.isUndefined(val) ? this.get(key) : this.set(key, val);
        },
        /**
         * @param {object|string} key
         * @param {*} [val]
         */
        set: function (key, val) {
            if (_.isObject(key)) {
                for (x in key) this.set(x, key[x]);
            } else {
                this._registry.setItem(key, val);
            }
        },
        /**
         * @param {string|int|array} key
         * @returns {object|string}
         */
        get: function (key) {
            var result;
            if (_.isArray(key)) {
                result = this._registry.getAll(key);
            } else if (_.isUndefined(key)) {
                result = this._registry.getAll();
            } else {
                result = this._registry.getItem(key);
            }
            return result;
        },
        /**
         * @param {array|string} [key]
         */
        remove: function (key) {
            if (_.isUndefined(key)) {
                return this._registry.clear();
            } else if (_.isArray(key)) {
                _.each(key, function (val) {
                    this._registry.removeItem(val);
                }, this)
                return;
            } else {
                return this._registry.removeItem(key);
            }
        },
        /**
         * 是否已注册
         * @param {string} key
         * @return bool
         */
        isRegistered: function (key) {
            return _.isString(key) && !_.isNull(this.get(key));
        }
    } );
    return Registry;
});
