define(function(){
	/*
	 * 构造函数
	 * @param countdownLabel 点击发送DOM元素 使用val属性设置提示信息
	 * @param phoneNo 手机号码
	 * @param certificationNo 证件号码
	 * @param certificationType 证件类型
	 * @param second 倒计时秒数
	 */
	var OTP = function(options){
		//倒计时秒数
		this.second = options.second || 60;
		this.countdown = this.second;
		this.interval = null;
		
		//倒计时标签
		this.countdownLabel = options.countdownLabel || null;
		
		this.phoneNo = options.phoneNo || null;
		this.certificationNo = options.certificationNo || null;
		this.certificationType = options.certificationType || null;
		
		this.lang = {
				awaitInput : '{second}秒后重发',
				buttonSend : '发送验证码',
				reSend : '点击重新发送'
			}
		
		this.init();
	}
	OTP.prototype.init = function(){
		if(this.countdownLabel != null){
			this.countdownLabel.val(this.lang.buttonSend);
		}
	}
    /*
     * 倒计时函数
     */
	OTP.prototype.countdownFun = function(){
		this.countdownLabel.attr('disabled', 'disabled');
		clearInterval(this.interval);
		this.countdown = this.second;
		this.timeLabel();
		this.interval = setInterval(_.bind(this.timeLabel, this), 1000);
	}
    /*
     * 停止计时
     */
    OTP.prototype.stop = function() {
        clearInterval(this.interval);
		this.countdownLabel.removeAttr('disabled');
		this.countdownLabel.val(this.lang.reSend);
    }
    /*
     * 显示倒计时
     */
	OTP.prototype.timeLabel = function(){
		if(this.countdown <= 0){
			this.stop();
			return;
		}
		var text = this.lang.awaitInput;
		text = text.replace('{second}', this.countdown --)
		this.countdownLabel.val(text);
	}
	/*
	 * 发送验证码
	 * @param successCallback 发送成功回调方法
	 * @param errorCallback发送失败回调方法
	 */
	OTP.prototype.send = function(successCallback, errorCallback){
		 var getCodeData = {
			mobile : this.phoneNo ,
			identity_no : this.certificationNo,
			identity_type : this.certificationType
		 };
		revokeInterface(
			App.getCreditCardUrl("padMessageTrigger"),
			"GET",
			true,
			"json",
			"data="+JSON.stringify(getCodeData).replace(/"/g, "'"),
			successCallback,
			errorCallback
		);
	}
	/*
	 * 短信验证
	 * @param verifyNo验证码
	 * @param successCallback验证成功回调方法
	 * @param errorCallback验证失败回调方法
	 */
	OTP.prototype.verify = function(verifyNo, successCallback, errorCallback){
		var verifyCodeData = {
			 mobile : this.phoneNo ,
			 identity_no : this.certificationNo,
			 identity_type : this.certificationType,
			 otp : verifyNo || ''
		}
		revokeInterface(App.getCreditCardUrl("padMessageInvalidate"),
				"GET",
				true,
				"json",
				"data="+JSON.stringify(verifyCodeData).replace(/"/g, "'"),
				successCallback,
				errorCallback
		);
	}
	return OTP;
})