define(['require', 'module', 'jquery'], function (require, module) {
    /**
     * 配置段管理
     * @param {String} [section]
     * @param {Object} [options]
     * @constructor
     */
    function Config(options) {
        var section, lastSec;
        options = options || {};
        section = options.section; $.isPlainObject(options) && $.extend(this._defaults, options.defaults);
        lastSec = Object.keys(this._defaults).pop();

        //通过当前加载域名判断配置段 todo 本地环境判断有无localhost和开发服务器兼容
        $.each(this._defaults, function(sec, opts) {
            if(opts.baseURL.match(new RegExp("^https?://" + location.hostname)) != null) {
                section = section || sec;
                //优先使用定义的配置段
                return false;
            }
        });
        this.section(section || lastSec, {
            config : options.config
        });
        window.DEBUG = this._config.debug = Boolean(this._config.debug && Boolean(this._config.section.indexOf(lastSec) == -1));
        return this;
    }

    function API(name) {
        //可使用$.mobile.path.parseUrl()进行替换$.url
        var regex = /^abs:\/*/, val = this[name] || "";
        return val.search(regex) != -1 ? val.replace(regex, "") : this.baseURL + val;
    }


    Config.prototype = {
        _defaults : {
            development : {},
            staging : {},
            product : {}
        },
        /**
         * 切换配置段
         * @param {string} section
         * @param {object} [opts]
         * @returns {object} config
         * @example
         * 选择配置段
         * this.section('development');
         * 覆盖指定配置（通常用于程序运行时临时改变配置）
         * this.section('development',{config:{api:{login:'login.php'}}});
         */
        section : function(section, opts) {
            //todo 自动追加API方法
            section = section || Object.keys(this._defaults)[0];
            opts = opts || {};

            //以继承方式合并到指定配置段
            var config = $.extend(true, {}, opts.config);
            for(var x in this._defaults) {
                config = $.extend(true, config, this._defaults[x]);
                if(x == section)
                    break;
            }
            config.section = section;
            this._config = this._config || {};
            $.extend(this._config, config);
            return config;
        },
        /**
         *
         * @param [name]
         * @returns {*}
         */
        get : function(name) {
            return this._config[name] || this._config;
        },
        set : function(name, value) {
            this._defaults[this._config.section][name] = value;
            this._config[name] = value;
            return this;
        }
    };
    var config = {
        //section: "dev",//如果设置，则视为强制设置，不会因为环境改变而改变。
        defaults: {
            development: {
                debug: 1,
                AjaxTimeout: 10000,
                positionTimeout: 60000,
                BMap: {
                    js: 'http://api.map.baidu.com/getscript?v=1.5&ak=fAvxGV9E8tF5tdRgz9avRxSw'
                },
                service: {
                    baseURL: 'http://10.13.50.210:8088/HMP/',
                    OTPSend: "hmotp/apply.do",
                    OTPValidate: "hmotp/validate.do",
                    instanceBackCall: "http://10.13.50.240:8080/HMP/instanceBackCall.do",
                    //cities: "http://stg.pingan.com.cn/cms-tmplt/wangdianchaxun/searchBankCityController.do",
                    //hotCities: "http://stg.pingan.com.cn/cms-tmplt/wangdianchaxun/searchWebIndexBankCityController.do",
                    //获取短信验证接口
                    "msgRequest": "userplatforms/hmotp/validate.do",
                    baiduMap: "lib/UI/BMap",
                    nearATM: "http://stg.pingan.com.cn/cms-tmplt/wangdianchaxun/searchBankATMController.do",
                    nearBank: "http://stg.pingan.com.cn/cms-tmplt/wangdianchaxun/searchBankNetworkController.do",
                    cities: "app/modules/home/data/cities.json",
                    API: API
                },
                baseURL: 'http://10.13.50.210:8088/HMP/',
                API: API
            },//修改这块时必须知会所有人
            staging: {
                baseURL: "http://test-txt.pingan.com.cn:8080/",
                BMap: {
                    //ak申请人胡欣苇 <EX-HUXINWEI001@pingan.com.cn>
                    js: 'http://api.map.baidu.com/getscript?v=1.5&ak=fAvxGV9E8tF5tdRgz9avRxSw'
                },
                service: {
                    cities: "app/modules/home/data/cities.json",
                    hotCities: "http://www.pingan.com/cms-tmplt/wangdianchaxun/searchWebIndexBankCityController.do",
                    nearATM: "http://www.pingan.com/cms-tmplt/wangdianchaxun/searchBankATMController.do",
                    nearBank: "http://www.pingan.com/cms-tmplt/wangdianchaxun/searchBankNetworkController.do",
                    nearLife: "http://dmzstg1.pingan.com.cn/life_insurance/elis.pa18.ios.commonQuery.visit?channelType=01&jsonFlag=Y&requestid=com.palic.elis.pos.intf.biz.action.PosQueryAction.queryCounterInfo2",
                    selfserver: "http://public-txt.pingan.com.cn/HMP/rest/getFuncPoints?funcType=",
                    PACollection: "http://public-txt.pingan.com.cn/HMP/rest/getPingAnCollection?resource=",
                    propertyurl: "http://test-public-txt.pingan.com.cn:10080/HMP/rest/redOpenId?appid=10007&url=" + encodeURIComponent("http://epcis-pa18-dmzstg2.pingan.com.cn/property_insurance/pa18AutoInquiry/mobile/paMiniShop.do?weappNo=xxxPACK95511_06")
                }
            },
            product: {
                baseURL: 'http://public-txt.pingan.com.cn',
                BMap: {
                    //ak申请人谷晓强 <GUXIAOQIANG836@pingan.com.cn>
                    js: 'http://api.map.baidu.com/getscript?v=1.5&ak=YfNaryuLxBEuECvmyKj3yGeU'
                },
                service: {
                    cities: "http://uc-txt.pingan.com.cn:7080/userplatforms/rest/syscity/querySysCityRelation",
                    hotCities: "http://www.pingan.com/cms-tmplt/wangdianchaxun/searchWebIndexBankCityController.do",
                    nearATM: "http://www.pingan.com/cms-tmplt/wangdianchaxun/searchBankATMController.do",
                    nearBank: "http://www.pingan.com/cms-tmplt/wangdianchaxun/searchBankNetworkController.do",
                    nearLife: "http://www.pingan.com/life_insurance/elis.pa18.ios.commonQuery.visit?channelType=01&jsonFlag=Y&requestid=com.palic.elis.pos.intf.biz.action.PosQueryAction.queryCounterInfo2",
                    selfserver: "http://public-txt.pingan.com.cn/HMP/rest/getFuncPoints?funcType=",
                    PACollection: "http://public-txt.pingan.com.cn/HMP/rest/getPingAnCollection?resource=",
                    propertyurl: "http://public-txt.pingan.com.cn/HMP/rest/redOpenId?appid=10007&url=" + encodeURIComponent("http://www.pingan.com/property_insurance/pa18AutoInquiry/mobile/paMiniShop.do?weappNo=PACK95511_06")
                }
            }
        }
    };
    config = window.config = new Config(config), cfg = config.get();

    window.DEBUG = cfg.debug && Boolean(cfg.section.indexOf('product') == -1);
    !DEBUG && console.warn('DEBUG is %s', DEBUG);

    //检查APP getConfig方法是否能正常调用
    if ( App.IS_NATIVE && App.IS_TXT ) {
        App.getConfig(function(data) {//获取Native APP环境配置
            data = JSON.parse(data);
            var section = 'prd';
            switch (true) {
                case data["CONFIG_TAG"].indexOf("dev") > -1 :
                    section = "dev";
                    break;
                case data["CONFIG_TAG"].indexOf("stg") > -1 :
                    section = "stg";
                    break;
                case data["CONFIG_TAG"].search(/prd|uat/) > -1 :
                    section = "prd";
                    break;
            }
            config.section(section);
            window.DEBUG = cfg.debug && Boolean(cfg.section.indexOf('prd') == -1); !DEBUG && console.warn('DEBUG is %s', DEBUG);
        });
    }

    // 使用console并加入调试模式
    /*//重写错误处理
    var errHandler = function (err, show) {
        //console.log(err);
        var msg = '', isShow = _.isBoolean(show) ? show : true;

        switch (true) {
            case typeof err == 'string':
                msg = arguments.length == 3
                    ? '消息:'+ err +';\n文件:'+ arguments[1] +':'+ arguments[2] +';\n行数:'+ arguments[2]
                    : '消息:'+ err
                break;
            case !!err.requireType:
                msg = '错误消息:'+ err.message
                    +';\n错误类型:'+ err.requireType
                    +';\n载入模块:'+ err.requireModules.toString()
                    +';\n调用堆栈:'+ err.stack;
                break;
            case err instanceof Error:
                msg = '错误消息:'+ err.message
                    +';\n调用堆栈:'+ err.stack;
                break;
        }
        isShow && msg && console.error(msg);
        return msg;
    };
    if (!DEBUG) {
        window.onerror = errHandler;
        requirejs.onError = errHandler;
    }*/
    $(document).ajaxSend(function(event, jqXHR, ajaxOptions){
        //allow abort for Ajax request during anytime
        $(this).one('abort.ajax', ajaxOptions, function(e){
            console.info('abort request of URL:'+e.data.url);
            //jqXHR.abort('The request has been aborted from URL:'+e.data.url);
            jqXHR.abort();
        });
    });
    $(document).ajaxError(function(event, jqXHR, ajaxOptions, thrownError){
        console.error("status:"+ jqXHR.status +" statusText:"+ jqXHR.statusText +" thrown:"+ thrownError + " url:"+ ajaxOptions.url);
    });
    //所有ajax请求响应队列完成后触发
    $(document).ajaxStop(function(event){
        $(this).off('abort.ajax');//abort.ajax事件的callbacks没有存在的意义。
    });
    $.ajaxSetup({ timeout:cfg.AjaxTimeout });

    return config;//选定配置段;
});